@@ -1,11 +1,11 @@
 /*
- * lftp and utils
+ * lftp - file transfer program
  *
- * Copyright (c) 1996-2004 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,12 +14,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: ArgV.cc,v 1.18 2007/09/12 13:03:06 lav Exp $ */
-
 #include <config.h>
 
 #include "misc.h"
@@ -34,6 +31,9 @@ ArgV::ArgV(const char *a0, const char *args_c)
    for(char *t=strtok(args," "); t; t=strtok(NULL," "))
       Append(t);
 }
+ArgV::~ArgV()
+{
+}
 
 void ArgV::seek(int n)
 {
@@ -56,15 +56,17 @@ void ArgV::back()
       ind--;
 }
 
-char *ArgV::Combine(int start) const
+char *ArgV::Combine(int start,int end) const
 {
    xstring res("");
-   if(start>=Count())
+   if(!end)
+      end=Count();
+   if(start>=end)
       return res.borrow();
    for(;;)
    {
       res.append(getarg(start++));
-      if(start>=Count())
+      if(start>=end)
 	 return(res.borrow());
       res.append(' ');
    }
