@@ -17,12 +17,11 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: Bencode.h,v 1.1 2011/08/01 11:07:04 lav Exp $ */
-
 #ifndef BENCODE_H
 #define BENCODE_H
 
 #include "xmap.h"
+#include "buffer.h"
 
 class BeNode
 {
@@ -42,14 +41,45 @@ public:
 
    static BeNode *Parse(const char *s,int len,int *rest);
 
+   BeNode(const xstring& s);
    BeNode(const char *s,int l);
+   BeNode(const char *s);
    BeNode(xarray_p<BeNode> *l);
    BeNode(xmap_p<BeNode> *d);
    BeNode(long long);
    ~BeNode();
 
+   BeNode *lookup(const char *key) {
+      return dict.lookup(key);
+   }
+   BeNode *lookup(const char *key,be_type_t t) {
+      BeNode *n=dict.lookup(key);
+      if(n && n->type!=t)
+	 n=0;
+      return n;
+   }
+   const xstring& lookup_str(const char *key) {
+      BeNode *n=dict.lookup(key);
+      if(!n || n->type!=BE_STR)
+	 return xstring::null;
+      return n->str;
+   }
+   long long lookup_int(const char *key) {
+      BeNode *n=dict.lookup(key);
+      if(!n || n->type!=BE_INT)
+	 return 0;
+      return n->num;
+   }
+
+   int ComputeLength();
+   const xstring& Pack();
+   void Pack(xstring &buf);
+   void Pack(const SMTaskRef<IOBuffer> &buf);
+
    void Format(xstring &buf,int level);
    const char *Format();
+   void Format1(xstring &buf);
+   const char *Format1();
 
    static const char *TypeName(be_type_t t);
 };
