@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2013 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: CmdExec.cc,v 1.157 2011/06/10 09:36:43 lav Exp $ */
-
 #include <config.h>
 #include <stddef.h>
 #include <errno.h>
@@ -57,11 +55,15 @@ static ResType lftp_cmd_vars[] = {
    {"cmd:verify-host",		 "yes",	  ResMgr::BoolValidate,0},
    {"cmd:at-exit",		 "",	  0,0},
    {"cmd:at-exit-bg",		 "",	  0,0},
+   {"cmd:at-exit-fg",		 "",	  0,0},
+   {"cmd:at-background",	 "",	  0,0},
+   {"cmd:at-terminate",		 "",	  0,0},
    {"cmd:at-finish",		 "",	  0,0},
    {"cmd:at-queue-finish",	 "",	  0,0},
    {"cmd:fail-exit",		 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
    {"cmd:verbose",		 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
-   {"cmd:interactive",		 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
+   {"cmd:interactive",		 "auto",  ResMgr::TriBoolValidate,ResMgr::NoClosure},
+   {"cmd:show-status",		 "yes",	  ResMgr::BoolValidate,ResMgr::NoClosure},
    {"cmd:move-background",	 "yes",	  ResMgr::BoolValidate,ResMgr::NoClosure},
    {"cmd:move-background-detach","yes",	  ResMgr::BoolValidate,ResMgr::NoClosure},
    {"cmd:set-term-status",	 "no",	  ResMgr::BoolValidate,0},
@@ -69,6 +71,7 @@ static ResType lftp_cmd_vars[] = {
    {"cmd:trace",		 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
    {"cmd:parallel",		 "1",	  ResMgr::UNumberValidate,0},
    {"cmd:queue-parallel",	 "1",	  ResMgr::UNumberValidate,0},
+   {"cmd:cls-exact-time",	 "yes",	  ResMgr::BoolValidate,ResMgr::NoClosure},
    {0}
 };
 static ResDecls lftp_cmd_vars_register(lftp_cmd_vars);
@@ -320,6 +323,7 @@ void CmdExec::RemoveFeeder()
       queue_feeder=0;
    delete replace_value(feeder,feeder->prev);
    Reconfig(0);
+   SetInteractive();
 }
 
 void CmdExec::ReuseSavedSession()
@@ -537,7 +541,7 @@ int CmdExec::Do()
 	    return MOVED;
 	 }
       }
-      if(status_line && status_line->CanShowNow())
+      if(status_line && show_status && status_line->CanShowNow())
 	 ShowRunStatus(status_line);   // this is only for top level CmdExec.
       return m;
    }
@@ -581,7 +585,7 @@ int CmdExec::Do()
 	    return MOVED;
 	 }
       }
-      if(status_line && status_line->CanShowNow())
+      if(status_line && show_status && status_line->CanShowNow())
 	 ShowRunStatus(status_line);   // this is only for top level CmdExec.
       if(m != STALL || interactive || waiting_num >= max_waiting)
 	 return m;
@@ -607,7 +611,7 @@ try_get_cmd:
 	       last_bg=-1;
 	 }
 
-	 if(status_line)
+	 if(status_line && show_status)
 	 {
 	    const char *def_title = FormatPrompt(ResMgr::Query("cmd:default-title",getenv("TERM")));
 	    status_line->DefaultTitle(def_title);
@@ -684,8 +688,10 @@ try_get_cmd:
 	 feeder->Bg();
       break;
    }
-   if(args==0 || args->count()==0)
+   if(args==0 || args->count()==0) {
+      AtFinish();
       return MOVED;  // empty command
+   }
 
    if(interactive)
       session->DontSleep(); // We don't want to get a delay just after user
@@ -775,6 +781,7 @@ void CmdExec::init(LocalDirectory *c)
    background=false;
 
    interactive=false;
+   show_status=true;
    top_level=false;
    auto_terminate_in_bg=false;
    feeder=0;
@@ -919,10 +926,10 @@ void CmdExec::Reconfig(const char *name)
    verify_path_cached=ResMgr::QueryBool("cmd:verify-path-cached",c);
    verify_host=ResMgr::QueryBool("cmd:verify-host",c);
    verbose=ResMgr::QueryBool("cmd:verbose",0);
-   // only allow explicit setting of cmd:interactive to change interactiveness.
-   if(top_level && name && !strcmp(name,"cmd:interactive"))
-      SetInteractive(ResMgr::QueryBool("cmd:interactive",0));
    max_waiting=ResMgr::Query(queue_feeder?"cmd:queue-parallel":"cmd:parallel",c);
+   if(name && !strcmp(name,"cmd:interactive"))
+      SetInteractive();
+   show_status=ResMgr::QueryBool("cmd:show-status",0);
 }
 
 void CmdExec::pre_stdout()
@@ -946,6 +953,7 @@ void CmdExec::SetCmdFeeder(CmdFeeder *new_feeder)
    new_feeder->saved_buf.set(cmd_buf.Get());
    feeder=new_feeder;
    cmd_buf.Empty();
+   SetInteractive();
 }
 
 int CmdExec::AcceptSig(int sig)
@@ -1027,6 +1035,13 @@ void CmdExec::SetInteractive(bool i)
    }
    interactive=i;
 }
+void CmdExec::SetInteractive()
+{
+   if(!top_level)
+      return;
+   bool def=feeder?feeder->IsInteractive():false;
+   SetInteractive(ResMgr::QueryTriBool("cmd:interactive",0,def));
+}
 
 xstring& xstring::append_quoted(const char *str,int len)
 {
@@ -1106,6 +1121,21 @@ void CmdExec::AtExitBg()
    FeedCmd(ResMgr::Query("cmd:at-exit-bg",0));
    FeedCmd("\n");
 }
+void CmdExec::AtExitFg()
+{
+   FeedCmd(ResMgr::Query("cmd:at-exit-fg",0));
+   FeedCmd("\n");
+}
+void CmdExec::AtBackground()
+{
+   FeedCmd(ResMgr::Query("cmd:at-background",0));
+   FeedCmd("\n");
+}
+void CmdExec::AtTerminate()
+{
+   FeedCmd(ResMgr::Query("cmd:at-terminate",0));
+   FeedCmd("\n");
+}
 
 void CmdExec::EmptyCmds()
 {
