@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2011 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: CopyJob.cc,v 1.48 2011/04/29 04:58:27 lav Exp $ */
-
 #include <config.h>
 #include "CopyJob.h"
 #include "ArgV.h"
@@ -40,7 +38,11 @@ int CopyJob::Do()
       return STALL;
    if(c->Error())
    {
-      eprintf("%s: %s\n",op.get(),c->ErrorText());
+      const char *error=c->ErrorText();
+      const char *name=GetDispName();
+      if(!strstr(error,name))
+	 error=xstring::cat(name,": ",error,NULL);
+      eprintf("%s: %s\n",op.get(),error);
       done=true;
       return MOVED;
    }
