@@ -1,7 +1,7 @@
 /*
- * lftp and utils
+ * lftp - file transfer program
  *
- * Copyright (c) 1998-2000 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,17 +14,18 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: DummyProto.cc,v 1.11 2008/11/27 05:56:11 lav Exp $ */
+/* $Id: DummyProto.cc,v 1.12 2009/03/20 12:28:04 lav Exp $ */
 
 #include <config.h>
 
 #include "DummyProto.h"
 #include <stddef.h>
 
+DummyProto::DummyProto() {}
+DummyProto::~DummyProto() {}
 int DummyProto::Do() { return STALL; }
 int DummyProto::Done() { return NO_HOST; }
 const char *DummyProto::GetProto() const { return ""; }
