@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2011 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: FileAccess.cc,v 1.168 2011/03/25 14:35:41 lav Exp $ */
-
 #include <config.h>
 
 #include "FileAccess.h"
@@ -65,12 +63,12 @@ void FileAccess::Init()
    retries=0;
    opt_date=0;
    opt_size=0;
+   fileset_for_info=0;
    error_code=OK;
    saved_errno=0;
    mkdir_p=false;
    ascii=false;
    norest_manual=false;
-   array_ptr=array_cnt=0;
 
    entity_size=NO_SIZE;
    entity_date=NO_DATE;
@@ -194,8 +192,8 @@ void FileAccess::Close()
    mode=CLOSED;
    opt_date=0;
    opt_size=0;
-   array_for_info=0;
-   array_ptr=array_cnt=0;
+   fileset_for_info=0;
+   retries=0;
    entity_size=NO_SIZE;
    entity_date=NO_DATE;
    ascii=false;
@@ -365,12 +363,11 @@ void FileAccess::SetPasswordGlobal(const char *p)
    }
 }
 
-void FileAccess::GetInfoArray(struct fileinfo *info,int count)
+void FileAccess::GetInfoArray(FileSet *info)
 {
    Open(0,ARRAY_INFO);
-   array_for_info=info;
-   array_ptr=0;
-   array_cnt=count;
+   fileset_for_info=info;
+   fileset_for_info->rewind();
 }
 
 static void expand_tilde(xstring &path, const char *home, int i=0)
