@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: FileCopy.h,v 1.85 2010/05/20 13:53:15 lav Exp $ */
-
 /*
   classes defined here:
    FileCopy
@@ -169,7 +167,6 @@ protected:
       } state;
 
 private:
-   int max_buf;
    bool cont;
 
    xstring_c error_text;
@@ -190,23 +187,25 @@ private:
    Ref<Buffer> line_buffer;
    int  line_buffer_max;
 
+   bool CheckFileSizeAtEOF() const;
+
 protected:
    void RateAdd(int a);
    void RateReset();
    off_t bytes_count;
 
 public:
-   off_t GetPos();
-   off_t GetSize();
-   int  GetPercentDone();
-   const char *GetPercentDoneStr();
-   float GetRate();
-   const char *GetRateStr();
-   off_t GetBytesRemaining();
-   long GetETA() { return GetETA(GetBytesRemaining()); }
-   long GetETA(off_t b);
-   const char *GetETAStr();
-   const char *GetETAStrSFromTime(time_t t) { return rate_for_eta->GetETAStrSFromTime(t); }
+   off_t GetPos() const;
+   off_t GetSize() const;
+   int  GetPercentDone() const;
+   const char *GetPercentDoneStr() const;
+   float GetRate() const;
+   const char *GetRateStr() const;
+   off_t GetBytesRemaining() const;
+   long GetETA() const { return GetETA(GetBytesRemaining()); }
+   long GetETA(off_t b) const;
+   const char *GetETAStr() const;
+   const char *GetETAStrSFromTime(time_t t) const { return rate_for_eta->GetETAStrSFromTime(t); }
    const char *GetStatus();
    FgData *GetFgData(bool fg);
    pid_t GetProcGroup();
@@ -232,8 +231,8 @@ public:
    void FailIfCannotSeek() { fail_if_cannot_seek=true; }
    void SetRange(off_t s,off_t lim);
    void SetRangeLimit(off_t lim) { get->range_limit=lim; }
-   off_t GetRangeStart() { return get->range_start; }
-   off_t GetRangeLimit() { return get->range_limit; }
+   off_t GetRangeStart() const { return get->range_start; }
+   off_t GetRangeLimit() const { return get->range_limit; }
    void RemoveSourceLater() { remove_source_later=true; }
    void RemoveTargetFirst() { remove_target_first=true; put->Resume(); put->RemoveFile(); }
    void LineBuffered(int size=0x1000);
@@ -291,7 +290,7 @@ class FileCopyPeerFA : public FileCopyPeer
    int Put_LL(const char *buf,int size);
    int PutEOF_LL();
 
-   FileAccess::fileinfo info;
+   FileSet info;
 
    bool fxp;   // FXP (ftp<=>ftp copy) active
 
@@ -415,4 +414,21 @@ public:
    void UseColor(bool c=true) { if(dl) dl->UseColor(c); }
 };
 
+class FileCopyPeerMemory : public FileCopyPeer
+{
+private:
+   int max_size;
+
+public:
+   FileCopyPeerMemory(int m) : FileCopyPeer(PUT), max_size(m) {}
+   FileCopyPeerMemory(const xstring& s) : FileCopyPeer(GET), max_size(0) {
+      Put(s);
+      PutEOF();
+      size=s.length();
+      pos=0;
+   }
+   int Do();
+   bool Done() { return true; }
+};
+
 #endif
