@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: FileSet.cc,v 1.75 2011/04/29 04:58:27 lav Exp $ */
-
 #include <config.h>
 #include "FileSet.h"
 
@@ -50,7 +48,7 @@
 
 void  FileInfo::Merge(const FileInfo& f)
 {
-   if(strcmp(name,f.name))
+   if(strcmp(basename_ptr(name),basename_ptr(f.name)))
       return;
 // int sim=defined&f.defined;
    int dif=(~defined)&f.defined;
@@ -58,6 +56,8 @@ void  FileInfo::Merge(const FileInfo& f)
       SetMode(f.mode);
    if(dif&DATE || (defined&DATE && f.defined&DATE && f.date.ts_prec<date.ts_prec))
       SetDate(f.date,f.date.ts_prec);
+   if(dif&SIZE)
+      SetSize(f.size);
    if(dif&TYPE)
       SetType(f.filetype);
    if(dif&SYMLINK)
@@ -75,7 +75,7 @@ void FileInfo::SetUser(const char *u)
    if(u==user)
       return;
    user=StringPool::Get(u);
-   defined|=USER;
+   def(USER);
 }
 
 void FileInfo::SetGroup(const char *g)
@@ -83,7 +83,7 @@ void FileInfo::SetGroup(const char *g)
    if(g==group)
       return;
    group=StringPool::Get(g);
-   defined|=GROUP;
+   def(GROUP);
 }
 
 void FileSet::add_before(int pos,FileInfo *fi)
@@ -148,6 +148,10 @@ FileSet::~FileSet()
  * be a bit of a pain to implement. */
 FileSet::FileSet(FileSet const *set)
 {
+   if(!set) {
+      ind=0;
+      return;
+   }
    ind=set->ind;
    for(int i=0; i<set->fnum; i++)
       files.append(new FileInfo(*(set->files[i])));
@@ -244,6 +248,8 @@ void FileSet::Empty()
 
 void FileSet::SubtractSame(const FileSet *set,int ignore)
 {
+   if(!set)
+      return;
    for(int i=0; i<fnum; i++)
    {
       FileInfo *f=set->FindByName(files[i]->name);
@@ -254,6 +260,8 @@ void FileSet::SubtractSame(const FileSet *set,int ignore)
 
 void FileSet::SubtractAny(const FileSet *set)
 {
+   if(!set)
+      return;
    for(int i=0; i<fnum; i++)
       if(set->FindByName(files[i]->name))
 	 Sub(i--);
@@ -433,6 +441,15 @@ void FileSet::Count(int *d,int *f,int *s,int *o) const
    }
 }
 
+void FileSet::CountBytes(long long *b) const
+{
+   for(int i=0; i<fnum; i++)
+   {
+      if(files[i]->filetype==FileInfo::NORMAL && files[i]->Has(FileInfo::SIZE))
+	 (*b)+=files[i]->size;
+   }
+}
+
 /* assumes sorted by name. binary search for name, returning the first name
  * >= name; returns fnum if name is greater than all names. */
 int FileSet::FindGEIndByName(const char *name) const
@@ -634,6 +651,7 @@ void FileInfo::Init()
    size=NO_SIZE;
    nlinks=0;
    defined=0;
+   need=0;
    user=0; group=0;
    rank=0;
 }
@@ -793,7 +811,8 @@ FileInfo *FileInfo::parse_ls_line(const char *line_c,const char *tz)
       // it's size, so the previous was group:
       fi->SetGroup(group_or_size);
       long long size;
-      if(sscanf(t,"%lld",&size)==1)
+      int n;
+      if(sscanf(t,"%lld%n",&size,&n)==1 && t[n]==0)
 	 fi->SetSize(size);
       t = NEXT_TOKEN;
       if(!t)
@@ -803,7 +822,8 @@ FileInfo *FileInfo::parse_ls_line(const char *line_c,const char *tz)
    {
       // it was month, so the previous was size:
       long long size;
-      if(sscanf(group_or_size,"%lld",&size)==1)
+      int n;
+      if(sscanf(group_or_size,"%lld%n",&size,&n)==1 && group_or_size[n]==0)
 	 fi->SetSize(size);
    }
 
@@ -935,7 +955,7 @@ void FileInfo::MakeLongName()
    int w=20-strlen(usergroup);
    if(w<1)
       w=1;
-   char size_str[20];
+   char size_str[21];
    if(defined&SIZE)
       snprintf(size_str,sizeof(size_str),"%*lld",w,(long long)size);
    else
