@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2011 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: Filter.h,v 1.27 2008/11/27 05:56:15 lav Exp $ */
-
 #ifndef FILTER_H
 #define FILTER_H
 
@@ -134,6 +132,8 @@ public:
 class FileStream : public FDStream
 {
    int mode;
+   int create_mode;
+   bool do_lock;
 public:
    FileStream(const char *fname,int open_mode);
    ~FileStream();
@@ -145,6 +145,8 @@ public:
    int getfd();
    bool can_seek();
    off_t get_size();
+   void set_lock() { do_lock=true; }
+   void set_create_mode(int m) { create_mode=m; }
 };
 
 #endif /* FILTER_H */
