@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2011 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: FindJob.h,v 1.25 2011/04/29 04:58:27 lav Exp $ */
-
 #ifndef FINDJOB_H
 #define FINDJOB_H
 
@@ -119,12 +117,14 @@ class FinderJob_List : public FinderJob
 {
    SMTaskRef<IOBuffer> buf;
    Ref<ArgV> args;
+   bool long_listing;
 protected:
    prf_res ProcessFile(const char *d,const FileInfo *fi);
    void Finish();
 
 public:
    FinderJob_List(FileAccess *s,ArgV *a,FDStream *o);
+   void DoLongListing(bool yes=true) { long_listing=yes; }
 
    int Done() { return FinderJob::Done() && buf->Done(); }
 };
