@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2013 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: Http.cc,v 1.242 2011/04/29 04:58:27 lav Exp $ */
-
 #include <config.h>
 #include "trio.h"
 #include <unistd.h>
@@ -38,6 +36,7 @@
 #include "HttpDir.h"
 #include "misc.h"
 #include "buffer_ssl.h"
+#include "buffer_zlib.h"
 
 #include "ascii_ctype.h"
 
@@ -65,6 +64,8 @@ CDECL char *strptime(const char *buf, const char *format, struct tm *tm);
 #define EINPROGRESS -1
 #endif
 
+enum { CHUNK_SIZE_UNKNOWN=-1 };
+
 Http::Connection::Connection(int s,const char *c)
    : closure(c), sock(s)
 {
@@ -104,7 +105,7 @@ void Http::Init()
    array_send=0;
 
    chunked=false;
-   chunk_size=-1;
+   chunk_size=CHUNK_SIZE_UNKNOWN;
    chunk_pos=0;
    chunked_trailer=false;
 
@@ -115,6 +116,8 @@ void Http::Init()
    no_cache=false;
 
    use_propfind_now=true;
+   allprop="";
+   allprop_len=0;
 
    retry_after=0;
 
@@ -187,11 +190,13 @@ void Http::ResetRequestData()
    sent_eot=false;
    keep_alive=false;
    keep_alive_max=-1;
-   array_send=array_ptr;
+   array_send=fileset_for_info?fileset_for_info->curr_index():0;
    chunked=false;
-   chunk_size=-1;
+   chunk_size=CHUNK_SIZE_UNKNOWN;
    chunk_pos=0;
    chunked_trailer=false;
+   propfind=0;
+   inflate=0;
    seen_ranges_bytes=false;
 }
 
@@ -243,6 +248,8 @@ void Http::Send(const char *format,...)
    va_start(va,format);
    xstring& str=xstring::vformat(format,va);
    va_end(va);
+   if(str.length()==0)
+      return;
    LogSend(5,str);
    conn->send_buf->Put(str);
 }
@@ -406,28 +413,47 @@ bool Http::ModeSupported()
    abort(); // should not happen
 }
 
-void Http::DirFile(xstring& path,const char *ecwd,const char *efile)
+void Http::DirFile(xstring& path,const xstring& ecwd,const xstring& efile) const
 {
-   int base=path.length();
-   if(!strcmp(ecwd,"~") && !hftp)
-      ecwd="";
-   const char *sep=(last_char(ecwd)=='/'?"":"/");
-   if(efile[0]==0)
-      sep="";
-   const char *pre=(ecwd[0]=='/'?"":"/");
-   if(efile[0]=='/')
-      path.append(efile);
-   else if(efile[0]=='~')
-      path.vappend("/",efile,NULL);
-   else
-      path.vappend(pre,ecwd,sep,efile,NULL);
+   const int base=path.length();
 
+   if(efile[0]=='/') {
+      path.append(efile);
+   } else if(efile[0]=='~' || ecwd.length()==0 || (ecwd.eq("~") && !hftp)) {
+      path.append('/');
+      path.append(efile);
+   } else {
+      if(ecwd[0]!='/')
+	 path.append('/');
+      path.append(ecwd);
+      if(ecwd.last_char()!='/' && efile.length()>0)
+	 path.append('/');
+      path.append(efile);
+   }
+
+   // remove "/~" or "/~/"
    if(path[base+1]=='~' && path[base+2]==0)
       path.truncate(base+1);
    else if(path[base+1]=='~' && path[base+2]=='/')
       path.set_substr(base,2,"");
 }
 
+void Http::SendPropfind(const xstring& efile,int depth)
+{
+   SendMethod("PROPFIND",efile);
+   Send("Depth: %d\r\n",depth);
+   if(allprop_len>0)
+   {
+      Send("Content-Type: text/xml\r\n");
+      Send("Content-Length: %d\r\n",allprop_len);
+   }
+}
+void Http::SendPropfindBody()
+{
+   if(allprop_len>0)
+      Send("%s",allprop);
+}
+
 void Http::SendRequest(const char *connection,const char *f)
 {
    xstring efile;
@@ -466,12 +492,6 @@ void Http::SendRequest(const char *connection,const char *f)
    if(mode==CHANGE_DIR && new_cwd && !new_cwd->url)
       add_slash=!new_cwd->is_file;
 
-   const char *allprop=	// PROPFIND request
-      "<?xml version=\"1.0\" ?>"
-      "<propfind xmlns=\"DAV:\">"
-        "<allprop/>"
-      "</propfind>\r\n";
-
    xstring pfile;
    if(proxy && !https)
    {
@@ -531,6 +551,10 @@ void Http::SendRequest(const char *connection,const char *f)
 	 SendMethod(special==HTTP_MOVE?"MOVE":"COPY",efile);
 	 Send("Destination: %s\r\n",special_data.get());
 	 break;
+      case HTTP_PROPFIND:
+	 SendMethod("PROPFIND",efile);
+	 Send("Depth: 1\r\n"); // directory listing required
+	 break;
       case HTTP_NONE:
 	 abort(); // cannot happen
       }
@@ -600,12 +624,7 @@ void Http::SendRequest(const char *connection,const char *f)
       if(mode==CHANGE_DIR)
       {
 	 if(use_propfind_now)
-	 {
-	    SendMethod("PROPFIND",efile);
-	    Send("Depth: 0\r\n"); // no directory listing required
-	    Send("Content-Type: text/xml\r\n");
-	    Send("Content-Length: %d\r\n",(int)strlen(allprop));
-	 }
+	    SendPropfind(efile,0);
 	 else
 	    SendMethod("HEAD",efile);
       }
@@ -624,10 +643,7 @@ void Http::SendRequest(const char *connection,const char *f)
       }
       else if(mode==MP_LIST)
       {
-	 SendMethod("PROPFIND",efile);
-	 Send("Depth: 1\r\n"); // directory listing required
-	 Send("Content-Type: text/xml\r\n");
-	 Send("Content-Length: %d\r\n",(int)strlen(allprop));
+	 SendPropfind(efile,1);
 	 pos=0;
       }
       break;
@@ -640,7 +656,10 @@ void Http::SendRequest(const char *connection,const char *f)
       break;
 
    case ARRAY_INFO:
-      SendMethod("HEAD",efile);
+      if(use_propfind_now)
+	 SendPropfind(efile,0);
+      else
+	 SendMethod("HEAD",efile);
       break;
 
    case RENAME:
@@ -666,23 +685,32 @@ void Http::SendRequest(const char *connection,const char *f)
 	 Send("%s",special_data.get());
       entity_size=NO_SIZE;
    }
-   else if(mode==MP_LIST || (mode==CHANGE_DIR && use_propfind_now))
-   {
-      Send("%s",allprop);
-   }
+   else if(!xstrcmp(last_method,"PROPFIND"))
+      SendPropfindBody();
 
    keep_alive=false;
    chunked=false;
-   chunk_size=-1;
+   chunk_size=CHUNK_SIZE_UNKNOWN;
    chunk_pos=0;
    chunked_trailer=false;
+   inflate=0;
    no_ranges=false;
 
    conn->send_buf->SetPos(0);
 }
 
-void Http::SendArrayInfoRequest()
+int Http::SendArrayInfoRequest()
 {
+   // skip to next needed file
+   for(FileInfo *fi=fileset_for_info->curr(); fi; fi=fileset_for_info->next())
+      if(fi->need)
+	 break;
+   if(array_send<fileset_for_info->curr_index())
+      array_send=fileset_for_info->curr_index();
+
+   if(state!=CONNECTED)
+      return 0;
+
    int m=1;
    if(keep_alive && use_head)
    {
@@ -690,12 +718,22 @@ void Http::SendArrayInfoRequest()
       if(m==-1)
 	 m=100;
    }
-   while(array_send-array_ptr<m && array_send<array_cnt)
+   int req_count=0;
+   while(array_send-fileset_for_info->curr_index()<m
+   && array_send<fileset_for_info->count())
    {
-      SendRequest(array_send==array_cnt-1 ? 0 : "keep-alive",
-	 array_for_info[array_send].file);
-      array_send++;
+      FileInfo *fi=(*fileset_for_info)[array_send++];
+      if(fi->need==0)
+	 continue;
+      xstring *name=&fi->name;
+      if(fi->filetype==fi->DIRECTORY && name->last_char()!='/') {
+	 name=&xstring::get_tmp(*name);
+	 name->append('/');
+      }
+      SendRequest(array_send==fileset_for_info->count()-1 ? 0 : "keep-alive", *name);
+      req_count++;
    }
+   return req_count;
 }
 
 static const char *extract_quoted_header_value(const char *value)
@@ -733,15 +771,15 @@ void Http::HandleHeaderLine(const char *name,const char *value)
       if(bs<0) // try to workaround broken servers
 	 bs+=0x100000000LL;
       body_size=bs;
-      if(pos==0 && mode!=STORE && mode!=MAKE_DIR)
+      if(pos==0 && mode!=STORE && mode!=MAKE_DIR && !inflate)
 	 entity_size=body_size;
-      if(pos==0 && opt_size && H_20X(status_code))
+      if(pos==0 && opt_size && H_20X(status_code) && !inflate)
 	 *opt_size=body_size;
 
       if(mode==ARRAY_INFO && H_20X(status_code))
       {
-	 array_for_info[array_ptr].size=body_size;
-	 array_for_info[array_ptr].get_size=false;
+	 FileInfo *fi=fileset_for_info->curr();
+	 fi->SetSize(body_size);
 	 TrySuccess();
       }
       return;
@@ -772,14 +810,20 @@ void Http::HandleHeaderLine(const char *name,const char *value)
    }
    if(!strcasecmp(name,"Last-Modified"))
    {
+      if(!H_20X(status_code))
+	 return;
+
       time_t t=Http::atotm(value);
-      if(opt_date && H_20X(status_code))
+      if(t==ATOTM_ERROR)
+	 return;
+
+      if(opt_date)
 	 *opt_date=t;
 
-      if(mode==ARRAY_INFO && H_20X(status_code))
+      if(mode==ARRAY_INFO)
       {
-	 array_for_info[array_ptr].time=t;
-	 array_for_info[array_ptr].get_time=false;
+	 FileInfo *fi=fileset_for_info->curr();
+	 fi->SetDate(t,0);
 	 TrySuccess();
       }
       return;
@@ -822,13 +866,24 @@ void Http::HandleHeaderLine(const char *name,const char *value)
       if(!strcasecmp(value,"chunked"));
       {
 	 chunked=true;
-	 chunk_size=-1;	// to indicate "before first chunk"
+	 chunk_size=CHUNK_SIZE_UNKNOWN;	  // expecting first chunk
 	 chunk_pos=0;
 	 chunked_trailer=false;
       }
-      // handle gzip?
       return;
    }
+   if(!strcasecmp(name,"Content-Encoding"))
+   {
+      if(!strcasecmp(value,"gzip")) {
+	 // inflated size if unknown beforehand
+	 entity_size=NO_SIZE;
+	 if(opt_size)
+	    *opt_size=NO_SIZE;
+	 // start the inflation
+	 inflate=new DirectedBuffer(DirectedBuffer::GET);
+	 inflate->SetTranslator(new DataInflator());
+      }
+   }
    if(!strcasecmp(name,"Accept-Ranges"))
    {
       if(!strcasecmp(value,"none"))
@@ -934,6 +989,37 @@ int Http::Do()
    if(Error())
       return m;
 
+   if(propfind && mode==CHANGE_DIR)
+   {
+      if(propfind->Eof())
+      {
+	 const char *b;
+	 int len;
+	 propfind->Get(&b,&len);
+	 FileSet *fs=HttpListInfo::ParseProps(b,len,GetCwd());
+	 if(fs)
+	 {
+	    FileInfo *fi=fs->curr();
+	    if(fi && fi->Has(fi->TYPE))
+	    {
+	       new_cwd->is_file=(fi->filetype!=fi->DIRECTORY);
+	       fi->MakeLongName();
+	       LogNote(9,"%s",fi->longname.get());
+	    }
+	 }
+	 cwd.Set(new_cwd);
+	 cache->SetDirectory(this, "", !cwd.is_file);
+	 state=DONE;
+	 propfind=0;
+	 return MOVED;
+      }
+      if(propfind->Error()) {
+	 SetError(NO_FILE,propfind->ErrorText());
+	 propfind=0;
+	 return MOVED;
+      }
+   }
+
    switch(state)
    {
    case DISCONNECTED:
@@ -944,6 +1030,16 @@ int Http::Do()
 	 state=DONE;
 	 return MOVED;
       }
+      if(mode==ARRAY_INFO)
+      {
+	 // check if we have anything to request
+	 SendArrayInfoRequest();
+	 if(!fileset_for_info->curr())
+	 {
+	    state=DONE;
+	    return MOVED;
+	 }
+      }
       if(!hftp && mode==QUOTE_CMD && !special)
       {
       handle_quote_cmd:
@@ -955,6 +1051,8 @@ int Http::Do()
 	    special=HTTP_COPY;
 	 else if(file && !strncasecmp(file,"MOVE ",5))
 	    special=HTTP_MOVE;
+	 else if(file && !strncasecmp(file,"PROPFIND ",9))
+	    special=HTTP_PROPFIND;
 	 else
 	 {
 	    SetError(NOT_SUPP,0);
@@ -963,7 +1061,9 @@ int Http::Do()
 	 if(special)
 	 {
 	    // METHOD encoded_path data
-	    const char *scan=file+5;
+	    const char *scan=file;
+	    while(*scan && *scan!=' ')
+	       scan++;
 	    while(*scan==' ')
 	       scan++;
 	    file_url.set(https?"https://":"http://");
@@ -1144,13 +1244,17 @@ int Http::Do()
 	 state=DONE;
 	 return MOVED;
       }
-      LogNote(9,_("Sending request..."));
       if(mode==ARRAY_INFO)
       {
-	 SendArrayInfoRequest();
+	 if(SendArrayInfoRequest()==0) {
+	    // nothing to do
+	    state=DONE;
+	    return MOVED;
+	 }
       }
       else
       {
+	 LogNote(9,_("Sending request..."));
 	 SendRequest();
       }
 
@@ -1235,22 +1339,37 @@ int Http::Do()
 	       }
 	       if(mode==ARRAY_INFO)
 	       {
+		  if((status_code==400 || status_code==501)
+		  && !xstrcmp(last_method,"PROPFIND"))
+		  {
+		     ResMgr::Set("http:use-propfind",hostname,"no");
+		     use_propfind_now=false;
+		     try_time=0;
+		     Disconnect();
+		     return MOVED;
+		  }
 		  // we'll have to receive next header
 		  status.set(0);
 		  status_code=0;
-		  if(array_for_info[array_ptr].get_time)
-		     array_for_info[array_ptr].time=NO_DATE;
-		  if(array_for_info[array_ptr].get_size)
-		     array_for_info[array_ptr].size=NO_SIZE;
-		  if(++array_ptr>=array_cnt)
+		  for(;;)
 		  {
+		     // skip to next needed file
+		     FileInfo *fi=fileset_for_info->next();
+		     if(!fi || fi->need)
+			break;
+		  }
+		  if(!fileset_for_info->curr())
+		  {
+		     // received all requested info.
 		     state=DONE;
 		     return MOVED;
 		  }
 		  // we can avoid reconnection if server supports it.
 		  if(keep_alive && (keep_alive_max>1 || keep_alive_max==-1) && use_head)
 		  {
-		     SendArrayInfoRequest();
+		     state=CONNECTED;
+		     if(SendArrayInfoRequest()==0)
+			state=DONE;
 		  }
 		  else
 		  {
@@ -1481,10 +1600,15 @@ int Http::Do()
       }
       if(mode==CHANGE_DIR)
       {
-	 cwd.Set(new_cwd);
-	 cache->SetDirectory(this, "", !cwd.is_file);
-	 state=DONE;
-	 return MOVED;
+	 if(!xstrcmp(last_method,"PROPFIND"))
+	    propfind=new IOBufferFileAccess(this);
+	 else
+	 {
+	    cwd.Set(new_cwd);
+	    cache->SetDirectory(this, "", !cwd.is_file);
+	    state=DONE;
+	    return MOVED;
+	 }
       }
 
       LogNote(9,_("Receiving body..."));
@@ -1501,6 +1625,7 @@ int Http::Do()
       }
       state=RECEIVING_BODY;
       m=MOVED;
+      /*passthrough*/
    case RECEIVING_BODY:
       if(conn->recv_buf->Error() || conn->send_buf->Error())
 	 goto handle_buf_error;
@@ -1570,6 +1695,7 @@ void  Http::ClassInit()
 
 void Http::SuspendInternal()
 {
+   super::SuspendInternal();
    if(conn)
       conn->SuspendInternal();
 }
@@ -1577,6 +1703,7 @@ void Http::ResumeInternal()
 {
    if(conn)
       conn->ResumeInternal();
+   super::ResumeInternal();
 }
 
 int Http::Read(void *buf,int size)
@@ -1601,6 +1728,18 @@ int Http::Read(void *buf,int size)
    }
    return res;
 }
+void Http::_Skip(int to_skip)
+{
+   if(inflate) {
+      inflate->Skip(to_skip);
+   } else {
+      conn->recv_buf->Skip(to_skip);
+      if(chunked)
+	 chunk_pos+=to_skip;
+      bytes_received+=to_skip;
+   }
+   real_pos+=to_skip;
+}
 int Http::_Read(void *buf,int size)
 {
    const char *buf1;
@@ -1628,7 +1767,8 @@ get_again:
    }
    if(!chunked)
    {
-      if(body_size>=0 && bytes_received>=body_size)
+      if(body_size>=0 && bytes_received>=body_size
+      && (!inflate || inflate->Size()==0))
       {
 	 LogNote(9,_("Received all"));
 	 return 0; // all received
@@ -1639,14 +1779,14 @@ get_again:
 	 return 0;
       }
    }
-   if(size1==0)
+   if(size1==0 && (!inflate || inflate->Size()==0))
       return DO_AGAIN;
-   if(chunked)
+   if(chunked && size1>0)
    {
       if(chunked_trailer && state==RECEIVING_HEADER)
 	 return DO_AGAIN;
       const char *nl;
-      if(chunk_size==-1) // expecting first/next chunk
+      if(chunk_size==CHUNK_SIZE_UNKNOWN) // expecting first/next chunk
       {
 	 nl=(const char*)memchr(buf1,'\n',size1);
 	 if(nl==0)  // not yet
@@ -1664,6 +1804,7 @@ get_again:
 	 }
 	 conn->recv_buf->Skip(nl-buf1+1);
 	 chunk_pos=0;
+	 LogNote(9,"next chunk size: %ld",chunk_size);
 	 goto get_again;
       }
       if(chunk_size==0) // eof
@@ -1685,14 +1826,15 @@ get_again:
 	    return FATAL;
 	 }
 	 conn->recv_buf->Skip(2);
-	 chunk_size=-1;
+	 chunk_size=CHUNK_SIZE_UNKNOWN;
 	 goto get_again;
       }
       // ok, now we may get portion of data
       if(size1>chunk_size-chunk_pos)
 	 size1=chunk_size-chunk_pos;
    }
-   else
+
+   if(!chunked)
    {
       // limit by body_size.
       if(body_size>=0 && size1+bytes_received>=body_size)
@@ -1702,8 +1844,23 @@ get_again:
    int bytes_allowed=0x10000000;
    if(rate_limit)
       bytes_allowed=rate_limit->BytesAllowedToGet();
-   if(size1>bytes_allowed)
-      size1=bytes_allowed;
+
+   if(inflate) {
+      // do the inflation, if there are not enough inflated data
+      if(size1>bytes_allowed)
+	 size1=bytes_allowed;
+      if(inflate->Size()<size && size1>0) {
+	 inflate->PutTranslated(buf1,size1);
+	 conn->recv_buf->Skip(size1);
+	 if(chunked)
+	    chunk_pos+=size1;
+	 bytes_received+=size1;
+      }
+      inflate->Get(&buf1,&size1);
+   } else {
+      if(size1>bytes_allowed)
+	 size1=bytes_allowed;
+   }
    if(size1==0)
       return DO_AGAIN;
    if(norest_manual && real_pos==0 && pos>0)
@@ -1713,21 +1870,13 @@ get_again:
       off_t to_skip=pos-real_pos;
       if(to_skip>size1)
 	 to_skip=size1;
-      conn->recv_buf->Skip(to_skip);
-      real_pos+=to_skip;
-      bytes_received+=to_skip;
-      if(chunked)
-	 chunk_pos+=to_skip;
+      _Skip(to_skip);
       goto get_again;
    }
    if(size>size1)
       size=size1;
    memcpy(buf,buf1,size);
-   conn->recv_buf->Skip(size);
-   if(chunked)
-      chunk_pos+=size;
-   real_pos+=size;
-   bytes_received+=size;
+   _Skip(size);
    return size;
 }
 
@@ -1897,6 +2046,15 @@ void Http::Reconfig(const char *name)
 
    user_agent=ResMgr::Query("http:user-agent",c);
    use_propfind_now=(use_propfind_now && QueryBool("use-propfind",c));
+
+   allprop=	// PROPFIND request
+      "<?xml version=\"1.0\" ?>"
+      "<propfind xmlns=\"DAV:\">"
+        "<allprop/>"
+      "</propfind>\r\n";
+   if(!QueryBool("use-allprop",c))
+      allprop="";
+   allprop_len=strlen(allprop);
 }
 
 bool Http::SameSiteAs(const FileAccess *fa) const
@@ -2248,7 +2406,7 @@ check_end (const char *p)
    The routine should probably be even more forgiving (as recommended
    by RFC2068), but I do not have the time to write one.
 
-   Return the computed time_t representation, or -1 if all the
+   Return the computed time_t representation, or ATOTM_ERROR if all the
    schemes fail.
 
    Needless to say, what we *really* need here is something like
@@ -2292,7 +2450,7 @@ Http::atotm (const char *time_string)
      whitespace instead of just one (it works that way on all the
      systems I've tested it on).  */
 
-   time_t ut=-1;
+   time_t ut=ATOTM_ERROR;
 
    setlocale(LC_TIME,"C"); // we need english month and week day names
 
