@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: Http.h,v 1.55 2008/11/27 05:56:19 lav Exp $ */
-
 #ifndef HTTP_H
 #define HTTP_H
 
@@ -83,9 +81,9 @@ class Http : public NetAccess
 
    void SendMethod(const char *,const char *);
    const char *last_method;
-   enum { HTTP_NONE=0, HTTP_POST, HTTP_MOVE, HTTP_COPY } special;
+   enum { HTTP_NONE=0, HTTP_POST, HTTP_MOVE, HTTP_COPY, HTTP_PROPFIND } special;
    xstring special_data;
-   void DirFile(xstring& path,const char *ecwd,const char *efile);
+   void DirFile(xstring& path,const xstring& ecwd,const xstring& efile) const;
    void SendAuth();
    void SendCacheControl();
    void SendBasicAuth(const char *tag,const char *auth);
@@ -95,7 +93,9 @@ class Http : public NetAccess
       {
 	 SendRequest(connection,file);
       }
-   void SendArrayInfoRequest();
+   int SendArrayInfoRequest(); // returns count of sent requests
+   void SendPropfind(const xstring& efile,int depth);
+   void SendPropfindBody();
    int status_code;
    void HandleHeaderLine(const char *name,const char *value);
    void GetBetterConnection(int level);
@@ -137,6 +137,9 @@ class Http : public NetAccess
    off_t chunk_pos;
    bool chunked_trailer;
 
+   Ref<DirectedBuffer> inflate;
+   Ref<IOBuffer> propfind;
+
    bool no_ranges;
    bool seen_ranges_bytes;
 
@@ -144,10 +147,15 @@ class Http : public NetAccess
    bool no_cache_this;
 
    bool use_propfind_now;
+   const char *allprop;
+   int allprop_len;
+
+   long retry_after;
 
    const char *user_agent;
 
    int _Read(void *,int);  // does not update pos, rate_limit, retries, does not check state.
+   void _Skip(int to_skip); // skip in recv_buf or inflate, update real_pos
 
 protected:
    bool hftp;  // ftp over http proxy.
@@ -199,6 +207,7 @@ public:
    void Cleanup();
    void CleanupThis();
 
+   static const time_t ATOTM_ERROR = -1;
    static time_t atotm (const char *time_string);
 };
 
