@@ -1,11 +1,11 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1999-2005 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2013 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,12 +14,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: HttpDir.cc,v 1.101 2007/08/29 11:20:13 lav Exp $ */
-
 #include <config.h>
 #include "ascii_ctype.h"
 #include <assert.h>
@@ -70,6 +67,8 @@ static bool find_value(const char *scan,const char *more,const char *name,xstrin
       if(*scan=='"' || *scan=='\'')
 	 quote=*scan++;
 
+      if(match)
+	 store.set("");
       while(scan<more && (quote ? *scan!=quote : !is_ascii_space(*scan)))
       {
 	 if(match)
@@ -306,10 +305,10 @@ static bool try_netscape_proxy(file_info &info,const char *str)
    {
       if(!strcasecmp(size_unit,"bytes")
       || !strcasecmp(size_unit,"byte"))
-	 sprintf(info.size_str,"%lld",info.size);
+	 snprintf(info.size_str,sizeof(info.size_str),"%lld",info.size);
       else
       {
-	 sprintf(info.size_str,"%lld%s",info.size,size_unit);
+	 snprintf(info.size_str,sizeof(info.size_str),"%lld%s",info.size,size_unit);
 	 info.size=-1;
       }
       debug("Netscape-Proxy 2.53 listing matched");
@@ -396,7 +395,7 @@ static bool try_apache_unixlike(file_info &info,const char *buf,
    && -1!=(info.month=parse_month(info.month_name))
    && -1!=parse_year_or_time(year_or_time,&info.year,&info.hour,&info.minute))
    {
-      sprintf(info.size_str,"%lld",info.size);
+      snprintf(info.size_str,sizeof(info.size_str),"%lld",info.size);
       if(info.perms[0]=='d')
 	 info.is_directory=true;
       else if(info.perms[0]=='l')
@@ -442,7 +441,8 @@ static bool try_roxen(file_info &info,const char *str)
 	 size_mod_i=1024*1024*1024;
       if(size_mod_i)
       {
-	 sprintf(info.size_str,"%s%s",info.size_str,size_mod);
+	 const char *old_size_str=alloca_strdup(info.size_str);
+	 snprintf(info.size_str,sizeof(info.size_str),"%s%s",old_size_str,size_mod);
 	 debug("Roxen web server listing matched");
 	 return true;
       }
@@ -482,13 +482,12 @@ static bool try_squid_ftp(file_info &info,const char *str,char *str_with_tags)
       char *sym_link=ptr+13;
       ptr=strchr(sym_link,'"');
       if(!ptr)
-	 sym_link=0;
+	 info.sym_link.unset();
       else
       {
 	 *ptr=0;
-	 url::decode_string(sym_link);
+	 info.sym_link.set(url::decode(sym_link));
       }
-      info.sym_link.set(sym_link);
    }
    debug("squid ftp listing matched");
    return true;
@@ -517,7 +516,7 @@ static bool try_wwwoffle_ftp(file_info &info,const char *buf,
    && -1!=(info.month=parse_month(info.month_name))
    && -1!=parse_year_or_time(year_or_time,&info.year,&info.hour,&info.minute))
    {
-      sprintf(info.size_str,"%lld",info.size);
+      snprintf(info.size_str,sizeof(info.size_str),"%lld",info.size);
       if(info.perms[0]=='d')
 	 info.is_directory=true;
       else if(info.perms[0]=='l')
@@ -571,7 +570,7 @@ static bool try_csm_proxy(file_info &info,const char *str)
 
    if (status) {
       debug("csm_proxy listing matched");
-      sprintf(info.size_str, "%lld", info.size);
+      snprintf(info.size_str,sizeof(info.size_str),"%lld",info.size);
       if (has_additional_file_info && additional_file_info[0]) {
 	  if (!strncasecmp("Symbolic Link",additional_file_info,13)) {
 	      info.is_sym_link = true;
@@ -588,6 +587,50 @@ static bool try_csm_proxy(file_info &info,const char *str)
    return status;
 }
 
+// 2004-Oct-19 02:10:26	0.2K	application/octet-stream
+static bool try_lighttpd_listing(file_info &info,char *str_with_tags)
+{
+   info.clear();
+
+   if(str_with_tags[0]=='/') {
+      info.is_directory=true;
+      str_with_tags++;
+   }
+
+   const char *next=strstr(str_with_tags,"\"m\">");
+   if(!next)
+      return false;
+   next+=4;
+   const char *end=strchr(next,'<');
+   if(!end)
+      return false;
+   xstring datetime(next,end-next);
+
+   next=strstr(end,"\"s\">");
+   if(!next)
+      return false;
+   next+=4;
+   end=strchr(next,'<');
+   if(!end)
+      return false;
+   xstring size(next,end-next);
+
+   int n=sscanf(datetime,"%4d-%3s-%2d %2d:%2d:%2d",
+	       &info.year,info.month_name,&info.day,
+	       &info.hour,&info.minute,&info.second);
+   if(n!=6)
+      return false;
+
+   if(is_ascii_digit(size[0])) {
+      strncpy(info.size_str,size,sizeof(info.size_str));
+      info.size_str[sizeof(info.size_str)-1]=0;
+   }
+
+   debug("lighttpd listing matched");
+
+   return true;
+}
+
 // this procedure is highly inefficient in some cases,
 // esp. when it has to return for more data many times.
 static int parse_html(const char *buf,int buf_len,bool eof,const Ref<Buffer>& list,
@@ -705,9 +748,9 @@ static int parse_html(const char *buf,int buf_len,bool eof,const Ref<Buffer>& li
    if(hftp)
    {
       // workaround proxy bugs.
-      char *t=strstr(link_target,";type=");
+      const char *t=strstr(link_target,";type=");
       if(t && t[6] && t[7]=='/' && t[8]==0)
-	 *t=0;
+	 link_target.truncate(t-link_target);
       const char *p=link_target+url::path_index(link_target);
       if(p[0]=='/' && p[1]=='/')
 	 link_target.set_substr(p-link_target+1,1,"%2F");
@@ -873,13 +916,11 @@ parse_url_again:
       }
    }
 
-   char *type=strstr(link_target,";type=");
+   const char *type=strstr(link_target,";type=");
    if(type && type[6] && !type[7])
    {
-      type[0]=0;
-      if(!all_links || all_links->FindByName(link_target))
+      if(!all_links || all_links->FindByName(xstring::get_tmp(link_target,type-link_target)))
 	 return tag_len;
-      type[0]=';';
    }
 
    if(link_target.length()==0)
@@ -960,6 +1001,8 @@ parse_url_again:
 	 skip_len=eol-buf+eol_len;
 	 goto got_info;
       }
+      if(try_lighttpd_listing(info,str_with_tags) && info.validate())
+	 goto got_info;
       if(try_mini_proxy(info,str)		&& info.validate()) goto got_info;
       if(try_apache_unixlike(info,str,more,more1,info_string)
       && info.validate())
@@ -995,7 +1038,7 @@ parse_url_again:
 	 info.month=parse_month(info.month_name);
       if(info.month>=0)
       {
-	 sprintf(info.month_name,"%02d",info.month+1);
+	 snprintf(info.month_name,sizeof(info.month_name),"%02d",info.month+1);
 	 if(info.year==-1)
 	    info.year=guess_year(info.month,info.day,info.hour,info.minute);
       }
@@ -1111,10 +1154,10 @@ parse_url_again:
 info_done:
    if(set && link_target[0]!='/' && link_target[0]!='~')
    {
-      char *slash=strchr(link_target,'/');
+      const char *slash=strchr(link_target,'/');
       if(slash)
       {
-	 *slash=0;
+	 link_target.truncate(slash-link_target);
 	 info.is_directory=true;
       }
 
@@ -1181,9 +1224,9 @@ int HttpDirList::Do()
       if(mode==FA::RETRIEVE)
       {
 	 // strip file name, directory remains.
-	 char *slash=strrchr(curr_url->path,'/');
+	 const char *slash=strrchr(curr_url->path,'/');
 	 if(slash && slash>curr_url->path)
-	    *slash=0;
+	    curr_url->path.truncate(slash-curr_url->path);
       }
 
    retry:
@@ -1311,18 +1354,15 @@ HttpDirList::~HttpDirList()
 
 const char *HttpDirList::Status()
 {
-   static char s[256];
    if(ubuf && !ubuf->Eof() && session->IsOpen())
-   {
-      sprintf(s,_("Getting file list (%lld) [%s]"),
+      return xstring::format(_("Getting file list (%lld) [%s]"),
 		     (long long)session->GetPos(),session->CurrentStatus());
-      return s;
-   }
    return "";
 }
 
 void HttpDirList::SuspendInternal()
 {
+   super::SuspendInternal();
    if(ubuf)
       ubuf->SuspendSlave();
 }
@@ -1330,8 +1370,9 @@ void HttpDirList::ResumeInternal()
 {
    if(ubuf)
       ubuf->ResumeSlave();
+   super::ResumeInternal();
 }
-
+#undef super
 
 // HttpListInfo implementation
 FileSet *HttpListInfo::Parse(const char *b,int len)
@@ -1351,6 +1392,9 @@ FileSet *Http::ParseLongList(const char *b,int len,int *err) const
    if(err)
       *err=0;
 
+   if(len>5 && !strncmp(b,"<?xml",5))
+      return HttpListInfo::ParseProps(b,len,GetCwd());
+
    FileSet *set=new FileSet;
    ParsedURL prefix(GetConnectURL());
    xstring_c base_href;
