@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: HttpDirXML.cc,v 1.10 2008/11/27 05:56:20 lav Exp $ */
-
 #include <config.h>
 #include "HttpDir.h"
 #include "log.h"
@@ -122,7 +120,9 @@ static void chardata_handle(void *data, const char *chardata, int len)
    }
    else if(!strcmp(tag,"DAV:getlastmodified"))
    {
-      ctx->fi->SetDate(Http::atotm(s),0);
+      time_t tm=Http::atotm(s);
+      if(tm!=Http::ATOTM_ERROR)
+	 ctx->fi->SetDate(tm,0);
    }
    else if(!strcmp(tag,"DAV:creator-displayname"))
    {
