@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2011 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: Job.cc,v 1.52 2011/04/29 04:58:28 lav Exp $ */
-
 #include <config.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -269,8 +267,7 @@ void  Job::ListDoneJobs()
       if(scan->jobno>=0 && (scan->parent==this || scan->parent==0)
          && !scan->deleting && scan->Done())
       {
-	 fprintf(f,_("[%d] Done (%s)"),scan->jobno,
-	    scan->cmdline?scan->cmdline.get():"?");
+	 fprintf(f,_("[%d] Done (%s)"),scan->jobno,scan->GetCmdLine().get());
 	 const char *this_url=this->GetConnectURL();
 	 this_url=alloca_strdup(this_url); // save it from overwriting.
 	 const char *that_url=scan->GetConnectURL();
@@ -289,7 +286,7 @@ xstring& Job::FormatJobTitle(xstring& s,int indent,const char *suffix)
    s.append_padding(indent,' ');
    if(jobno>=0)
       s.appendf("[%d] ",jobno);
-   s.append(cmdline?cmdline.get():"?");
+   s.append(GetCmdLine());
    if(suffix) {
       s.append(' ');
       s.append(suffix);
@@ -381,6 +378,7 @@ xstring& SessionJob::FormatStatus(xstring& s,int v,const char *prefix)
    if(url && *url) {
       s.append(prefix);
       s.append(url);
+      s.append('\n');
    }
    return s;
 }
