@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: Job.h,v 1.37 2011/04/29 04:58:28 lav Exp $ */
-
 #ifndef JOB_H
 #define JOB_H
 
@@ -76,8 +74,7 @@ public:
    virtual void	ShowRunStatus(const SMTaskRef<StatusLine>&);
    void ClearStatus()
       {
-	 const char *empty="";
-	 eprintf(empty);
+	 eprintf("%s",""); /* just "" causes a -Wformat-zero-length" warning */
       }
    virtual void	  SayFinal() {}; // final phrase of fg job
    virtual int	  Done()=0;
@@ -89,6 +86,7 @@ public:
 
    xstring cmdline;
 
+   virtual const xstring& GetCmdLine() const { return cmdline?cmdline:xstring::get_tmp("?",1); }
    xstring& FormatJobTitle(xstring& s,int indent=0,const char *suffix=0);
    xstring& FormatOneJob(xstring& s,int verbose,int indent=0,const char *suffix=0);
    xstring& FormatOneJobRecursively(xstring& s,int verbose,int indent=0);
