@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: LocalDir.cc,v 1.5 2008/11/27 05:56:21 lav Exp $ */
-
 #include <config.h>
 
 #include <stdlib.h>
@@ -38,6 +36,7 @@ void LocalDirectory::SetFromCWD()
 {
    Unset();
    fd=open(".",O_RDONLY|O_DIRECTORY);
+   fcntl(fd,F_SETFD,FD_CLOEXEC);
    name.set_allocated(xgetcwd());
 }
 
@@ -83,8 +82,10 @@ LocalDirectory::LocalDirectory()
 LocalDirectory::LocalDirectory(const LocalDirectory *o)
 {
    fd=-1;
-   if(o->fd!=-1)
+   if(o->fd!=-1) {
       fd=dup(o->fd);
+      fcntl(fd,F_SETFD,FD_CLOEXEC);
+   }
    name.set(o->name);
 }
 
