@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: MirrorJob.cc,v 1.178 2011/04/29 04:58:28 lav Exp $ */
-
 #include <config.h>
 
 #include <sys/types.h>
@@ -549,6 +547,13 @@ void  MirrorJob::InitSets(const FileSet *source,const FileSet *dest)
       to_transfer->SubtractAny(to_rm_mismatched);
 
    to_transfer->SortByPatternList(ResMgr::Query("mirror:order",0));
+   to_transfer->CountBytes(&bytes_to_transfer);
+   if(parent_mirror)
+      parent_mirror->AddBytesToTransfer(bytes_to_transfer);
+
+   int dir_count=0;
+   to_transfer->Count(&dir_count,NULL,NULL,NULL);
+   only_dirs = (dir_count==to_transfer->count());
 }
 
 void MirrorJob::HandleChdir(FileAccessRef& session, int &redirections)
@@ -793,7 +798,8 @@ int   MirrorJob::Do()
       if(FlagSet(DEPTH_FIRST) && source_set && !target_set)
       {
 	 // transfer directories first
-	 to_transfer=new FileSet(source_set);
+	 InitSets(source_set,NULL);
+	 to_transfer->Unsort();
 	 to_transfer->SubtractNotDirs();
 	 goto pre_WAITING_FOR_TRANSFER;
       }
@@ -817,7 +823,9 @@ int   MirrorJob::Do()
    case(WAITING_FOR_TRANSFER):
       while((j=FindDoneAwaitedJob())!=0)
       {
-	 stats.bytes+=j->GetBytesCount();
+	 long long bytes_count=j->GetBytesCount();
+	 AddBytesTransferred(bytes_count);
+	 stats.bytes+=bytes_count;
 	 stats.time +=j->GetTimeSpent();
 	 if(j->ExitCode()!=0)
 	    stats.error_count++;
@@ -843,8 +851,7 @@ int   MirrorJob::Do()
 
 	       // if we have not created any subdirs and there are only subdirs,
 	       // then the directory would be empty - skip it.
-	       if(FlagSet(NO_EMPTY_DIRS) && stats.dirs==0
-	       && source_set->get_fnum()==to_transfer->get_fnum())
+	       if(FlagSet(NO_EMPTY_DIRS) && stats.dirs==0 && only_dirs)
 		  goto pre_FINISHING;
 
 	       transfer_count+=root_transfer_count;
@@ -1099,6 +1106,7 @@ MirrorJob::MirrorJob(MirrorJob *parent,
    FileAccess *source,FileAccess *target,
    const char *new_source_dir,const char *new_target_dir)
  :
+   bytes_transferred(0), bytes_to_transfer(0),
    source_dir(new_source_dir), target_dir(new_target_dir),
    root_transfer_count(0),
    transfer_count(parent?parent->transfer_count:root_transfer_count)
@@ -1315,6 +1323,49 @@ void MirrorJob::SetOnChange(const char *oc)
    on_change.set(oc);
 }
 
+const char *MirrorJob::AddPattern(Ref<PatternSet>& exclude,char opt,const char *optarg)
+{
+   PatternSet::Type type=
+      (opt=='x'||opt=='X'||opt=='\0'?PatternSet::EXCLUDE:PatternSet::INCLUDE);
+   PatternSet::Pattern *pattern=0;
+   if(opt=='x' || opt=='i')
+   {
+      Ref<PatternSet::Regex> rx(new PatternSet::Regex(optarg));
+      if(rx->Error())
+	 return xstring::get_tmp(rx->ErrorText());
+      pattern=rx.borrow();
+   }
+   else if(opt=='X' || opt=='I')
+   {
+      pattern=new PatternSet::Glob(optarg);
+   }
+   if(!exclude)
+   {
+      const char *default_exclude=ResMgr::Query("mirror:exclude-regex",0);
+      const char *default_include=ResMgr::Query("mirror:include-regex",0);
+
+      // don't create default pattern set if not needed
+      if(!pattern && !(default_exclude && *default_exclude))
+	 return NULL;
+
+      exclude=new PatternSet;
+      /* Make default_exclude the first pattern so that it can be
+       * overridden by --include later, and do that only when first
+       * explicit pattern is for exclusion - otherwise all files are
+       * excluded by default and no default exclusion is needed. */
+      if(type==PatternSet::EXCLUDE && default_exclude && *default_exclude)
+      {
+	 exclude->Add(type,new PatternSet::Regex(default_exclude));
+	 if(default_include && *default_include)
+	    exclude->Add(PatternSet::INCLUDE,new PatternSet::Regex(default_include));
+      }
+   }
+   if(pattern)
+      exclude->Add(type,pattern);
+
+   return NULL; // no error
+}
+
 CMD(mirror)
 {
 #define args (parent->args)
@@ -1363,6 +1414,7 @@ CMD(mirror)
       {"reverse",no_argument,0,'R'},
       {"verbose",optional_argument,0,'v'},
       {"newer-than",required_argument,0,'N'},
+      {"file",required_argument,0,'f'},
       {"older-than",required_argument,0,OPT_OLDER_THAN},
       {"size-range",required_argument,0,OPT_SIZE_RANGE},
       {"dereference",no_argument,0,'L'},
@@ -1388,6 +1440,8 @@ CMD(mirror)
       {"no-empty-dirs",no_argument,0,OPT_NO_EMPTY_DIRS},
       {"depth-first",no_argument,0,OPT_DEPTH_FIRST},
       {"ascii",no_argument,0,OPT_ASCII},
+      {"target-directory",required_argument,0,'O'},
+      {"destination-directory",required_argument,0,'O'},
       {0}
    };
 
@@ -1415,16 +1469,17 @@ CMD(mirror)
    const char *on_change=0;
 
    Ref<PatternSet> exclude;
-   const char *default_exclude=ResMgr::Query("mirror:exclude-regex",0);
-   const char *default_include=ResMgr::Query("mirror:include-regex",0);
 
    if(!ResMgr::QueryBool("mirror:set-permissions",0))
       flags|=MirrorJob::NO_PERMS;
    if(ResMgr::QueryBool("mirror:dereference",0))
       flags|=MirrorJob::RETR_SYMLINKS;
 
+   const char *source_dir=NULL;
+   const char *target_dir=NULL;
+
    args->rewind();
-   while((opt=args->getopt_long("esi:x:I:X:nrpcRvN:LP:a",mirror_opts,0))!=EOF)
+   while((opt=args->getopt_long("esi:x:I:X:nrpcRvN:LP:af:O:",mirror_opts,0))!=EOF)
    {
       switch(opt)
       {
@@ -1460,43 +1515,13 @@ CMD(mirror)
       case('X'):
       case('I'):
       {
-	 PatternSet::Type type=
-	    (opt=='x'||opt=='X'?PatternSet::EXCLUDE:PatternSet::INCLUDE);
-	 PatternSet::Pattern *pattern=0;
-	 if(opt=='x' || opt=='i')
+	 const char *err=MirrorJob::AddPattern(exclude,opt,optarg);
+	 if(err)
 	 {
-	    Ref<PatternSet::Regex> rx(new PatternSet::Regex(optarg));
-	    if(rx->Error())
-	    {
-	       eprintf(_("%s: regular expression `%s': %s\n"),
-		  args->a0(),optarg,rx->ErrorText());
-	       goto no_job;
-	    }
-	    pattern=rx.borrow();
+	    eprintf(_("%s: regular expression `%s': %s\n"),
+	       args->a0(),optarg,err);
+	    goto no_job;
 	 }
-	 else // X or I
-	 {
-	    pattern=new PatternSet::Glob(optarg);
-	 }
-	 if(!exclude)
-	 {
-	    exclude=new PatternSet;
-	    /* Make default_exclude the first pattern so that it can be
-	     * overridden by --include later, and do that only when first
-	     * explicit pattern is for exclusion - otherwise all files are
-	     * excluded by default and no default exclusion is needed. */
-	    if(type==PatternSet::EXCLUDE && default_exclude && *default_exclude)
-	    {
-	       exclude->Add(type,new PatternSet::Regex(default_exclude));
-	       if(default_include && *default_include)
-		  exclude->Add(PatternSet::INCLUDE,new PatternSet::Regex(default_include));
-	    }
-	    default_exclude=0;
-	    /* Users usually don't want to exclude all directories */
-	    if(type==PatternSet::INCLUDE)
-	       exclude->Add(type,new PatternSet::Regex("/$"));
-	 }
-	 exclude->Add(type,pattern);
 	 break;
       }
       case('R'):
@@ -1519,6 +1544,18 @@ CMD(mirror)
       case('N'):
 	 newer_than=optarg;
 	 break;
+      case('f'):
+      {
+	 // mirror for a single file (or glob pattern).
+	 flags|=MirrorJob::NO_RECURSION;
+	 MirrorJob::AddPattern(exclude,'I',basename_ptr(optarg));
+	 source_dir=dirname(optarg);
+	 source_dir=alloca_strdup(source_dir); // save the temp string
+	 break;
+      }
+      case('O'):
+	 target_dir=optarg;
+	 break;
       case(OPT_OLDER_THAN):
 	 older_than=optarg;
 	 break;
@@ -1606,23 +1643,28 @@ CMD(mirror)
       }
    }
 
-   /* add default exclusion if no explicit patterns were specified */
-   if(!exclude && default_exclude && *default_exclude)
+   if(exclude && !(flags&MirrorJob::NO_RECURSION))
    {
-      exclude=new PatternSet;
-      exclude->Add(PatternSet::EXCLUDE,new PatternSet::Regex(default_exclude));
-      if(default_include && *default_include)
-	 exclude->Add(PatternSet::INCLUDE,new PatternSet::Regex(default_include));
+      /* Users usually don't want to exclude all directories when recursing */
+      if(exclude->GetFirstType()==PatternSet::INCLUDE)
+	 exclude->AddFirst(PatternSet::INCLUDE,new PatternSet::Regex("/$"));
    }
 
-   args->back();
+   /* add default exclusion if no explicit patterns were specified */
+   if(!exclude)
+      MirrorJob::AddPattern(exclude,'\0',0);
 
-   const char *source_dir=".";
-   const char *target_dir=".";
+   args->back();
 
    const char *arg=args->getnext();
    if(arg)
    {
+      if(source_dir)
+      {
+	 eprintf(_("%s: ambiguous source directory (`%s' or `%s'?)\n"),args->a0(),
+	    source_dir,arg);
+	 goto no_job;
+      }
       source_dir=arg;
       ParsedURL source_url(source_dir);
       if(source_url.proto && source_url.path)
@@ -1639,6 +1681,12 @@ CMD(mirror)
       arg=args->getnext();
       if(arg)
       {
+	 if(target_dir)
+	 {
+	    eprintf(_("%s: ambiguous target directory (`%s' or `%s'?)\n"),args->a0(),
+	       target_dir,arg);
+	    goto no_job;
+	 }
 	 target_dir=arg;
 	 ParsedURL target_url(target_dir);
 	 if(target_url.proto && target_url.path)
@@ -1674,6 +1722,11 @@ CMD(mirror)
       }
    }
 
+   if(!source_dir)
+      source_dir=".";
+   if(!target_dir)
+      target_dir=".";
+
    if(!reverse)
    {
       if(!source_session)
