@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: MirrorJob.h,v 1.57 2011/04/29 04:58:28 lav Exp $ */
-
 #ifndef MIRRORJOB_H
 #define MIRRORJOB_H
 
@@ -26,6 +24,7 @@
 #include "FileSet.h"
 #include "Job.h"
 #include "PatternSet.h"
+#include "misc.h"
 
 class MirrorJob : public Job
 {
@@ -51,6 +50,8 @@ class MirrorJob : public Job
    bool target_is_local;
    bool source_is_local;
 
+   long long bytes_transferred;
+   long long bytes_to_transfer;
    Ref<FileSet> target_set;
    Ref<FileSet> source_set;
    Ref<FileSet> to_transfer;
@@ -60,6 +61,27 @@ class MirrorJob : public Job
    Ref<FileSet> old_files_set;
    Ref<FileSet> new_files_set;
    void	 InitSets(const FileSet *src,const FileSet *dst);
+   bool only_dirs;
+
+   void AddBytesTransferred(long long b) {
+      bytes_transferred+=b;
+      if(parent_mirror)
+	 parent_mirror->AddBytesTransferred(b);
+   }
+   void AddBytesToTransfer(long long b) {
+      bytes_to_transfer+=b;
+      if(parent_mirror)
+	 parent_mirror->AddBytesToTransfer(b);
+   }
+
+   const xstring& GetCmdLine() const {
+      if(bytes_to_transfer>0) {
+	 return xstring::get_tmp(cmdline).appendf(" - %lld/%lld (%d%%)",
+	    bytes_transferred,bytes_to_transfer,
+	    percent(bytes_transferred,bytes_to_transfer));
+      }
+      return cmdline;
+   }
 
    void	 HandleFile(FileInfo *);
 
@@ -209,6 +231,7 @@ public:
       }
    void SetMaxErrorCount(int ec) { max_error_count=ec; }
    void SetOnChange(const char *oc);
+   static const char *AddPattern(Ref<PatternSet>& exclude,char opt,const char *optarg);
 };
 
 #endif//MIRRORJOB_H
