@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2011 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: NetAccess.cc,v 1.112 2011/06/16 11:08:42 lav Exp $ */
-
 #include <config.h>
 
 #include <errno.h>
@@ -343,6 +341,7 @@ bool NetAccess::CheckRetries()
       return false;
    }
    try_time=now;
+   LogNote(10,"attempt number %d",retries);
    return true;
 }
 void NetAccess::TrySuccess()
@@ -416,7 +415,21 @@ do_again:
    if(done)
       return m;
 
-   if(!ubuf && !get_info)
+   if(session->OpenMode()==FA::ARRAY_INFO)
+   {
+      res=session->Done();
+      if(res==FA::DO_AGAIN)
+	 return m;
+      if(res==FA::IN_PROGRESS)
+	 return m;
+      session->Close();
+      FileAccess::cache->UpdateFileSet(session,"",FA::MP_LIST,result);
+      FileAccess::cache->UpdateFileSet(session,"",FA::LONG_LIST,result);
+      done=true;
+      return MOVED;
+   }
+
+   if(!ubuf)
    {
       const char *cache_buffer=0;
       int cache_buffer_size=0;
@@ -523,19 +536,16 @@ got_fileset:
       if(!result)
 	 result=new FileSet;
 
-      result->ExcludeDots();
       result->Exclude(exclude_prefix,exclude);
       result->rewind();
       for(file=result->curr(); file!=0; file=result->next())
       {
-	 FA::fileinfo add;
-	 add.size=NO_SIZE;
-	 add.time=NO_DATE;
-	 add.get_size = need_size && !(file->defined & file->SIZE);
-	 add.get_time = need_time && (!(file->defined & file->DATE)
-				 || (file->date.ts_prec>0 && can_get_prec_time));
-	 add.is_dir=false;
-	 add.file=0;
+	 file->need=0;
+	 if(need_size && !file->Has(file->SIZE))
+	    file->Need(file->SIZE);
+	 if(need_time && (!file->Has(file->DATE)
+                          || (file->date.ts_prec>0 && can_get_prec_time)))
+	    file->Need(file->DATE);
 
 	 if(file->defined & file->TYPE)
 	 {
@@ -543,61 +553,27 @@ got_fileset:
 	    {
 	       //file->filetype=file->NORMAL;
 	       file->defined &= ~(file->SIZE|file->SYMLINK_DEF|file->MODE|file->DATE|file->TYPE);
-	       add.get_size=true;
-	       add.get_time=true;
+	       file->Need(file->SIZE|file->DATE);
 	    }
-
-	    if(file->filetype==file->SYMLINK)
+	    else if(file->filetype==file->SYMLINK)
 	    {
 	       // don't need these for symlinks
-	       add.get_size=false;
-	       add.get_time=false;
+	       file->NoNeed(file->SIZE|file->DATE);
+	       // but need the link target
+	       if(!file->Has(file->SYMLINK_DEF))
+		  file->Need(file->SYMLINK_DEF);
 	    }
 	    else if(file->filetype==file->DIRECTORY)
 	    {
 	       if(!get_time_for_dirs)
 		  continue;
 	       // don't need size for directories
-	       add.get_size=false;
-	       add.is_dir=true;
+	       file->NoNeed(file->SIZE);
 	    }
 	 }
-
-	 if(add.get_size || add.get_time)
-	 {
-	    add.file=file->name;
-	    if(!add.get_size)
-	       add.size=NO_SIZE;
-	    if(!add.get_time)
-	       add.time=NO_DATE;
-	    get_info.append(add);
-	 }
       }
-      if(get_info.count()==0)
-      {
-	 done=true;
-	 return m;
-      }
-      session->GetInfoArray(get_info.get_non_const(),get_info.count());
-   }
-   if(get_info)
-   {
-      res=session->Done();
-      if(res==FA::DO_AGAIN)
-	 return m;
-      if(res==FA::IN_PROGRESS)
-	 return m;
-      session->Close();
-      for(int i=0; i<get_info.count(); i++)
-      {
-	 if(get_info[i].time!=NO_DATE)
-	    result->SetDate(get_info[i].file,get_info[i].time,0);
-	 if(get_info[i].size!=NO_SIZE)
-	    result->SetSize(get_info[i].file,get_info[i].size);
-      }
-      get_info.unset();
-      done=true;
-      m=MOVED;
+      session->GetInfoArray(result.get_non_const());
+      session->Roll();
    }
    return m;
 }
@@ -616,7 +592,7 @@ const char *GenericParseListInfo::Status()
       return xstring::format("%s (%lld) %s[%s]",_("Getting directory contents"),
 		     (long long)session->GetPos(),
 		     ubuf->GetRateStrS(),session->CurrentStatus());
-   if(get_info)
+   if(session->OpenMode()==FA::ARRAY_INFO)
       return xstring::format("%s (%d%%) [%s]",_("Getting files information"),
 		     session->InfoArrayPercentDone(),
 		     session->CurrentStatus());
