@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2011 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: PatternSet.cc,v 1.6 2008/11/27 05:56:23 lav Exp $ */
-
 #include <config.h>
 #include <fnmatch.h>
 #include "PatternSet.h"
@@ -26,11 +24,28 @@
 PatternSet::PatternSet()
 {
    chain=0;
+   first=0;
 }
 void PatternSet::Add(Type t,Pattern *p)
 {
    chain=new PatternLink(t,p,chain);
+   if(!first)
+      first=chain;
 }
+
+void PatternSet::AddFirst(Type t,Pattern *p)
+{
+   PatternLink *new_link=new PatternLink(t,p,NULL);
+   if(!first)
+      first=chain=new_link;
+   else
+      first->next=new_link;
+}
+PatternSet::Type PatternSet::GetFirstType() const
+{
+   return first->type;
+}
+
 PatternSet::~PatternSet()
 {
    while(chain)
