@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: PatternSet.h,v 1.5 2008/11/27 05:56:24 lav Exp $ */
-
 #ifndef EXCLUDE_H
 #define EXCLUDE_H
 
@@ -63,12 +61,15 @@ private:
    };
 
    PatternLink *chain;
+   PatternLink *first;
 
 public:
    PatternSet();
    ~PatternSet();
 
    void	Add(Type,Pattern *);
+   void	AddFirst(Type,Pattern *);
+   Type GetFirstType() const;
 
    bool Match(Type t,const char *str) const;
    bool MatchExclude(const char *str) const { return Match(EXCLUDE,str); }
