@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2011 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: SFtp.h,v 1.32 2011/03/16 09:03:09 lav Exp $ */
-
 #ifndef SFTP_H
 #define SFTP_H
 
@@ -54,8 +52,8 @@ enum packet_type {
    SSH_FXP_RMDIR    =15,
    SSH_FXP_REALPATH =16,
    SSH_FXP_STAT     =17,
-   SSH_FXP_RENAME   =18,
-   SSH_FXP_READLINK =19,
+   SSH_FXP_RENAME   =18,   // v>=2
+   SSH_FXP_READLINK =19,   // v>=3
    SSH_FXP_SYMLINK  =20,   // v<=5
    SSH_FXP_LINK     =21,   // v>=6
    SSH_FXP_BLOCK    =22,   // v>=6
@@ -648,6 +646,11 @@ private:
       void ComputeLength();
       void Pack(Buffer *b);
    };
+   class Request_READLINK : public PacketSTRING
+   {
+   public:
+      Request_READLINK(const char *name) : PacketSTRING(SSH_FXP_READLINK,name) {}
+   };
    class Request_SYMLINK : public Packet
    {
       xstring oldpath;
@@ -691,6 +694,7 @@ private:
 	 HANDLE_STALE,
 	 DATA,
 	 INFO,
+	 INFO_READLINK,
 	 DEFAULT,
 	 WRITE_STATUS,
 	 IGNORE
@@ -741,6 +745,7 @@ private:
    void	 RequestMoreData();
    off_t request_pos;
 
+   void MergeAttrs(FileInfo *fi,const FileAttrs *a);
    FileInfo *MakeFileInfo(const NameAttrs *a);
 
    int max_packets_in_flight;
