@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: SleepJob.cc,v 1.46 2011/06/24 05:24:13 lav Exp $ */
-
 #include <config.h>
 #include <ctype.h>
 #include <stddef.h>
@@ -119,7 +117,7 @@ void SleepJob::ShowRunStatus(const SMTaskRef<StatusLine>& s)
       Job::ShowRunStatus(s);
    else
    {
-      s->Show(Status());
+      s->Show("%s",Status());
       current->TimeoutS(1);
    }
 }
