@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1999-2000 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,12 +14,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: Speedometer.h,v 1.10 2009/08/18 05:25:23 lav Exp $ */
-
 #ifndef SPEEDOMETER_H
 #define SPEEDOMETER_H
 
@@ -40,6 +37,7 @@ public:
    float Get();
    float Get() const { return rate; }
    static xstring& GetStr(float r);
+   static xstring& GetStrProper(float r);
    xstring& GetStr();
    static const char *GetStrS(float r);
    const char *GetStrS();
