@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2011 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: StatusLine.cc,v 1.44 2011/04/29 04:58:28 lav Exp $ */
-
 #include <config.h>
 #include <sys/types.h>
 #ifdef HAVE_TERMIOS_H
@@ -59,6 +57,10 @@ int  StatusLine::GetWidth()
 
 StatusLine::StatusLine(int new_fd)
 {
+   to_status_line = get_string_term_cap("tsl", "ts");
+   from_status_line = get_string_term_cap("fsl", "fs");
+   prev_line = get_string_term_cap("cuu1","up");
+
    fd=new_fd;
    update_delayed=false;
    next_update_title_only=false;
@@ -131,9 +133,9 @@ void StatusLine::ShowN(const char *const* newstr,int n)
    }
 }
 
-const char *StatusLine::to_status_line = get_string_term_cap("tsl", "ts");
-const char *StatusLine::from_status_line = get_string_term_cap("fsl", "fs");
-const char *StatusLine::prev_line = get_string_term_cap("cuu1","up");
+const char *StatusLine::to_status_line;
+const char *StatusLine::from_status_line;
+const char *StatusLine::prev_line;
 
 void StatusLine::WriteTitle(const char *s, int fd) const
 {
