@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2011 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -80,8 +80,8 @@ class TimeDate : public Time
    void set_local_time();
 
 public:
-   TimeDate() { SetToCurrentTime(); local_time_unix=0; }
-   TimeDate(time_t s,int ms=0) : Time(s,ms) { local_time_unix=0; }
+   TimeDate() { SetToCurrentTime(); local_time_unix=(time_t)-1; }
+   TimeDate(time_t s,int ms=0) : Time(s,ms) { local_time_unix=(time_t)-1; if(local_time_unix==s) --local_time_unix; }
 
    operator const struct tm *() { set_local_time(); return &local_time; }
    operator const struct tm &() { set_local_time(); return local_time; }
