@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2013 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: attach.h,v 1.5 2011/08/01 10:32:05 lav Exp $ */
-
 #ifndef ATTACH_H
 #define ATTACH_H
 
@@ -31,7 +29,6 @@
 #include <fcntl.h>
 #include <locale.h>
 #include <ctype.h>
-#include <sys/utsname.h>
 #include <sys/un.h>
 #if HAVE_SYS_SOCKET_H
 # include <sys/socket.h>
@@ -53,13 +50,17 @@ class AcceptTermFD : public SMTask
 {
    int sock;
    int a_sock;
+   int recv_i;
+   int fds[3];
    bool accepted;
    bool detached;
 public:
-   AcceptTermFD() : sock(-1), a_sock(-1), accepted(false), detached(false) {
+   AcceptTermFD() : sock(-1), a_sock(-1), recv_i(0), accepted(false), detached(false) {
       do_listen();
    }
    ~AcceptTermFD() {
+      for(int i=0; i<recv_i; i++)
+	 close(fds[i]);
       if(sock!=-1) {
 	 close(sock);
 	 unlink(get_sock_path());
@@ -69,6 +70,8 @@ public:
    }
    int Do() {
       int m=STALL;
+      if(detached)
+	 return m;
       if(accepted) {
 	 char buf;
 	 int res=read(a_sock,&buf,1);
@@ -111,27 +114,31 @@ public:
 	 fcntl(a_sock,F_SETFD,FD_CLOEXEC);
 	 m=MOVED;
       }
-      int fd=recvfd(a_sock,0);
-      if(fd==-1 && E_RETRY(errno)) {
-	 Block(a_sock,POLLIN);
-	 return m;
+      while(recv_i<=2) {
+	 int fd=recvfd(a_sock,0);
+	 if(fd==-1 && E_RETRY(errno)) {
+	    Block(a_sock,POLLIN);
+	    return m;
+	 }
+	 if(fd==-1)
+	 {
+	    perror("recvfd");
+	    do_listen();
+	    TimeoutS(1);
+	    return m;
+	 }
+	 fcntl(fd,F_SETFD,FD_CLOEXEC);
+	 fds[recv_i]=fd;
+	 ++recv_i;
       }
-      if(fd==-1)
-      {
-	 perror("recvfd");
-	 do_listen();
-	 TimeoutS(1);
-	 return m;
-      }
-      fcntl(fd,F_SETFD,FD_CLOEXEC);
-      printf(_("[%u] Attached to terminal %s. %s\n"),(unsigned)getpid(),ttyname(fd),now.IsoDateTime());
+      printf(_("[%u] Attached to terminal %s. %s\n"),(unsigned)getpid(),ttyname(fds[1]),now.IsoDateTime());
       fflush(stdout);
       fflush(stderr);
-      dup2(fd,0);
-      dup2(fd,1);
-      dup2(fd,2);
-      if(fd>2)
-	 close(fd);
+      for(int i=0; i<recv_i; i++) {
+	 dup2(fds[i],i);
+	 if(fds[i]>=recv_i)
+	    close(fds[i]);
+      }
       close(sock);
       sock=-1;
       unlink(get_sock_path());
@@ -142,11 +149,9 @@ public:
    static xstring& get_sock_path(int pid=0) {
       if(!pid)
 	 pid=getpid();
-      struct utsname u;
-      uname(&u);
-      const char *home=get_lftp_home();
+      const char *home=get_lftp_data_dir();
       mkdir(xstring::format("%s/bg",home),0700);
-      return xstring::format("%s/bg/%s-%d",home,u.nodename,pid);
+      return xstring::format("%s/bg/%s-%d",home,get_nodename(),pid);
    }
    void do_listen() {
       const char *path=get_sock_path();
@@ -157,6 +162,9 @@ public:
 	 close(a_sock);
 	 a_sock=-1;
       }
+      for(int i=0; i<recv_i; i++)
+	 close(fds[i]);
+      recv_i=0;
       accepted=false;
       detached=false;
       sock=socket(AF_UNIX,SOCK_STREAM,0);
@@ -193,9 +201,10 @@ class SendTermFD : public SMTask
    int sock;
    bool connected;
    bool sent;
+   int send_i;
    bool detached;
 public:
-   SendTermFD(pid_t p) : pid(p), sock(-1), connected(false), sent(false), detached(false) {}
+   SendTermFD(pid_t p) : pid(p), sock(-1), connected(false), sent(false), send_i(0), detached(false) {}
    ~SendTermFD() {
       if(sock>=0)
 	 close(sock);
@@ -204,6 +213,24 @@ public:
       int m=STALL;
       if(error || detached)
 	 return m;
+      if(sent) {
+	 char buf;
+	 int res=read(sock,&buf,1);
+	 if(res==-1 && E_RETRY(errno)) {
+	    Block(sock,POLLIN);
+	    return m;
+	 }
+	 if(res<=0) {
+	    detached=true;
+	    close(sock);
+	    sock=-1;
+	    SignalHook::DoCount(SIGINT);
+	    SignalHook::Restore(SIGQUIT);
+	    SignalHook::DoCount(SIGTSTP);
+	    SignalHook::Restore(SIGWINCH);
+	 }
+	 return MOVED;
+      }
       if(sock==-1) {
 	 sock=socket(AF_UNIX,SOCK_STREAM,0);
 	 if(sock==-1) {
@@ -239,8 +266,8 @@ public:
 	 connected=true;
 	 m=MOVED;
       }
-      if(!sent) {
-	 if(sendfd(sock,1)<0) {
+      while(send_i<=2) {
+	 if(sendfd(sock,send_i)<0) {
 	    if(NonFatalError(errno)) {
 	       Block(sock,POLLOUT);
 	       return m;
@@ -250,30 +277,17 @@ public:
 	    sock=-1;
 	    return 0;
 	 }
-	 sent=true;
+	 ++send_i;
 	 m=MOVED;
-
-	 pass_pid=pid;
+      }
+      sent=true;
+      pass_pid=pid;
+      if(isatty(0)) {
 	 SignalHook::Handle(SIGINT,pass_sig);
 	 SignalHook::Handle(SIGQUIT,pass_sig);
 	 SignalHook::Handle(SIGTSTP,pass_sig);
 	 SignalHook::Handle(SIGWINCH,pass_sig);
       }
-      char buf;
-      int res=read(sock,&buf,1);
-      if(res==-1 && E_RETRY(errno)) {
-	 Block(sock,POLLIN);
-	 return m;
-      }
-      if(res<=0) {
-	 detached=true;
-	 close(sock);
-	 sock=-1;
-	 SignalHook::DoCount(SIGINT);
-	 SignalHook::Restore(SIGQUIT);
-	 SignalHook::DoCount(SIGTSTP);
-	 SignalHook::Restore(SIGWINCH);
-      }
       return MOVED;
    }
    bool Done() { return error || detached; }
