@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2013 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -243,6 +243,20 @@ void DataTranslator::AppendTranslated(Buffer *target,const char *put_buf,int siz
    target->SetPos(old_pos);
 }
 
+void DirectedBuffer::SetTranslator(DataTranslator *t)
+{
+   if(mode==GET && !translator && Size()>0) {
+      // translate unread data
+      const char *data;
+      int len;
+      Get(&data,&len);
+      t->Put(data,len);
+      buffer.truncate(buffer_ptr);
+      t->AppendTranslated(this,0,0);
+   }
+   translator=t;
+}
+
 #ifdef HAVE_ICONV
 void DataRecoder::PutTranslated(Buffer *target,const char *put_buf,int size)
 {
@@ -342,6 +356,8 @@ void DirectedBuffer::SetTranslation(const char *enc,bool translit)
       return;
    const char *from_code=(mode==GET?enc:local_code);
    const char *to_code  =(mode==GET?local_code:enc);
+   if(!strcasecmp(from_code,to_code))
+      return;
    SetTranslator(new DataRecoder(from_code,to_code,translit));
 }
 #endif //HAVE_ICONV
@@ -384,7 +400,7 @@ void DirectedBuffer::EmbraceNewData(int len)
 
 
 IOBuffer::IOBuffer(dir_t m)
-   : DirectedBuffer(m), event_time(now)
+   : DirectedBuffer(m), event_time(now), max_buf(0)
 {
 }
 IOBuffer::~IOBuffer()
@@ -528,6 +544,12 @@ int IOBufferStacked::Put_LL(const char *buf,int size)
 
 int IOBufferStacked::Get_LL(int)
 {
+   if(max_buf && Size()>=max_buf) {
+      down->SuspendSlave();
+      return 0;
+   }
+   down->ResumeSlave();
+
    const char *b;
    int size;
    down->Get(&b,&size);
@@ -550,6 +572,18 @@ bool IOBufferStacked::Done()
    return false;
 }
 
+void IOBufferStacked::SuspendInternal()
+{
+   super::SuspendInternal();
+   down->SuspendSlave();
+}
+void IOBufferStacked::ResumeInternal()
+{
+   if(!max_buf || Size()<max_buf)
+      down->ResumeSlave();
+   super::ResumeInternal();
+}
+
 // IOBufferFDStream implementation
 #include <fcntl.h>
 #include <unistd.h>
@@ -608,6 +642,9 @@ stream_err:
 
 int IOBufferFDStream::Get_LL(int size)
 {
+   if(max_buf && Size()>=max_buf)
+      return 0;
+
    int res=0;
 
    int fd=stream->getfd();
@@ -667,6 +704,12 @@ IOBufferFDStream::~IOBufferFDStream() {}
 #define super IOBuffer
 int IOBufferFileAccess::Get_LL(int size)
 {
+   if(max_buf && Size()>=max_buf) {
+      session->SuspendSlave();
+      return 0;
+   }
+   session->ResumeSlave();
+
    int res=0;
 
    res=session->Read(GetSpace(size),size);
@@ -684,11 +727,14 @@ int IOBufferFileAccess::Get_LL(int size)
 
 void IOBufferFileAccess::SuspendInternal()
 {
+   super::SuspendInternal();
    session->SuspendSlave();
 }
 void IOBufferFileAccess::ResumeInternal()
 {
-   session->ResumeSlave();
+   if(!max_buf || Size()<max_buf)
+      session->ResumeSlave();
+   super::ResumeInternal();
 }
 const char *IOBufferFileAccess::Status()
 {
