@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2013 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: commands.cc,v 1.308 2011/06/16 12:24:05 lav Exp $ */
-
 #include <config.h>
 
 #include "modconfig.h"
@@ -172,6 +170,7 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	     * exception; they both seem to be options used manually, so I made
 	     * them align with GNU ls options. */
 	    " -1                   - single-column output\n"
+	    " -a, --all            - show dot files\n"
 	    " -B, --basename       - show basename of files only\n"
 	    "     --block-size=SIZ - use SIZ-byte blocks\n"
 	    " -d, --directory      - list directory entries instead of contents\n"
@@ -250,7 +249,7 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
    {"get",     cmd_get,    N_("get [OPTS] <rfile> [-o <lfile>]"),
 	 N_("Retrieve remote file <rfile> and store it to local file <lfile>.\n"
 	 " -o <lfile> specifies local file name (default - basename of rfile)\n"
-	 " -c  continue, reget\n"
+	 " -c  continue, resume transfer\n"
 	 " -E  delete remote files after successful transfer\n"
 	 " -a  use ascii mode (binary is the default)\n"
 	 " -O <base> specifies base directory or URL where files should be placed\n")},
@@ -266,8 +265,9 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 " -a  all types\n")},
    {"help",    cmd_help,   N_("help [<cmd>]"),
 	 N_("Print help for command <cmd>, or list of available commands\n")},
-   {"jobs",    cmd_jobs,   "jobs [-v]",
-	 N_("List running jobs. -v means verbose, several -v can be specified.\n")},
+   {"jobs",    cmd_jobs,   "jobs [-v] [<job_no...>]",
+	 N_("List running jobs. -v means verbose, several -v can be specified.\n"
+	    "If <job_no> is specified, only list a job with that number.\n")},
    {"kill",    cmd_kill,   N_("kill all|<job_no>"),
 	 N_("Delete specified job with <job_no> or all jobs\n")},
    {"lcd",     cmd_lcd,    N_("lcd <ldir>"),
@@ -297,7 +297,7 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 "See also `help cls'.\n")},
    {"mget",    cmd_get,	   N_("mget [OPTS] <files>"),
 	 N_("Gets selected files with expanded wildcards\n"
-	 " -c  continue, reget\n"
+	 " -c  continue, resume transfer\n"
 	 " -d  create directories the same as in file names and get the\n"
 	 "     files into them instead of current directory\n"
 	 " -E  delete remote files after successful transfer\n"
@@ -604,6 +604,7 @@ Job *CmdExec::builtin_cd()
       dir=args->getarg(1);
    }
 
+   bool dir_needs_slash=false;
    if(url::is_url(dir))
    {
       ParsedURL u(dir,true);
@@ -614,18 +615,23 @@ Job *CmdExec::builtin_cd()
 	 return builtin_open();
       url=dir;
       dir=alloca_strdup(u.path);
-      if(url::dir_needs_trailing_slash(u.proto))
-	 is_file=(last_char(dir)!='/');
+      dir_needs_slash=url::dir_needs_trailing_slash(u.proto);
    }
    else
    {
-      if(url::dir_needs_trailing_slash(session->GetProto()))
-	 is_file=(last_char(dir)!='/');
+      dir_needs_slash=url::dir_needs_trailing_slash(session->GetProto());
    }
 
+   if(dir_needs_slash)
+      is_file=(last_char(dir)!='/');
+
    int cache_is_dir=FileAccess::cache->IsDirectory(session,dir);
    if(cache_is_dir==1)
+   {
+      if(is_file && dir_needs_slash)
+	 dir=xstring::get_tmp(dir).append('/');
       is_file=false;
+   }
    else if(cache_is_dir==0)
       is_file=true;
 
@@ -789,7 +795,6 @@ Job *CmdExec::builtin_lftp()
    {
       SetCmdFeeder(lftp_feeder);
       lftp_feeder=0;
-      SetInteractive(isatty(0));
       FeedCmd("||command exit\n");   // if the command fails, quit
    }
 
@@ -1402,6 +1407,7 @@ const char *FileSetOutput::parse_argv(const Ref<ArgV>& a)
       OPT_USER
    };
    static struct option cls_options[] = {
+      {"all",no_argument,0,'a'},
       {"basename",no_argument,0,'B'},
       {"directory",no_argument,0,'d'},
       {"human-readable",no_argument,0,'h'},
@@ -1431,7 +1437,7 @@ const char *FileSetOutput::parse_argv(const Ref<ArgV>& a)
    const char *time_style=ResMgr::Query("cmd:time-style",0);
 
    int opt;
-   while((opt=a->getopt_long(":1BdFhiklqsDISrt", cls_options))!=EOF)
+   while((opt=a->getopt_long(":a1BdFhiklqsDISrt", cls_options))!=EOF)
    {
       switch(opt) {
       case OPT_SORT:
@@ -1471,6 +1477,9 @@ const char *FileSetOutput::parse_argv(const Ref<ArgV>& a)
 	 if(output_block_size == 0)
 	    return _("invalid block size");
 	 break;
+      case('a'):
+	 showdots = true;
+	 break;
       case('1'):
 	 single_column = true;
          break;
@@ -1528,6 +1537,8 @@ const char *FileSetOutput::parse_argv(const Ref<ArgV>& a)
 
    time_fmt.set(0);
    if(time_style && time_style[0]) {
+      if (mode & DATE)
+	 need_exact_time=ResMgr::QueryBool("cmd:cls-exact-time",0);
       if(time_style[0]=='+')
 	 time_fmt.set(time_style+1);
       else if(!strcmp(time_style,"full-iso"))
@@ -1539,20 +1550,6 @@ const char *FileSetOutput::parse_argv(const Ref<ArgV>& a)
 	 time_fmt.set("%Y-%m-%d \n%m-%d %H:%M");
       else
 	 time_fmt.set(time_style);
-      need_exact_time=false;
-      if(time_fmt) {
-	 static const char exact_fmts[][3]={"%H","%M","%S","%N",""};
-	 int sep=strcspn(time_fmt,"\n|");
-	 for(int i=0; exact_fmts[i][0]; i++) {
-	    const char *f=strstr(time_fmt,exact_fmts[i]);
-	    if(!f)
-	       continue;
-	    if(i>1 || sep>f-time_fmt) {
-	       need_exact_time=true;
-	       break;
-	    }
-	 }
-      }
    }
 
    // remove parsed options.
@@ -1920,7 +1917,7 @@ CMD(jobs)
    const char *arg=args->getnext();
    xstring s("");
    if(!arg) {
-      parent->FormatJobs(s,v);
+      parent->top->FormatJobs(s,v);
    } else {
       for(; arg; arg=args->getnext()) {
 	 if(!isdigit((unsigned char)*arg)) {
@@ -2587,7 +2584,7 @@ CMD(help)
 CMD(ver)
 {
    printf(
-      _("LFTP | Version %s | Copyright (c) 1996-%d Alexander V. Lukyanov\n"),VERSION,2012);
+      _("LFTP | Version %s | Copyright (c) 1996-%d Alexander V. Lukyanov\n"),VERSION,2013);
    printf("\n");
    printf(
       _("LFTP is free software: you can redistribute it and/or modify\n"
@@ -2853,13 +2850,15 @@ CMD(find)
    static struct option find_options[]=
    {
       {"maxdepth",required_argument,0,'d'},
+      {"ls",no_argument,0,'l'},
       {0,0,0,0}
    };
    int opt;
    int maxdepth = -1;
+   bool long_listing=false;
    const char *op=args->a0();
 
-   while((opt=args->getopt_long("+d:",find_options))!=EOF)
+   while((opt=args->getopt_long("+d:l",find_options))!=EOF)
    {
       switch(opt)
       {
@@ -2871,6 +2870,9 @@ CMD(find)
 	 }
 	 maxdepth = atoi(optarg);
 	 break;
+      case 'l':
+	 long_listing=true;
+	 break;
       case '?':
 	 eprintf(_("Usage: %s [-d #] dir\n"),op);
 	 return 0;
@@ -2881,6 +2883,7 @@ CMD(find)
       args->Append(".");
    FinderJob_List *j=new class FinderJob_List(session->Clone(),args.borrow(),output.borrow());
    j->set_maxdepth(maxdepth);
+   j->DoLongListing(long_listing);
    return j;
 }
 
