@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: complete.cc,v 1.84 2011/06/16 11:09:32 lav Exp $ */
-
 #include <config.h>
 
 #include "trio.h"
@@ -394,6 +392,7 @@ static completion_type cmd_completion_type(const char *cmd,int start)
    || !strcmp(buf,"ls")
    || !strcmp(buf,"cls")
    || !strcmp(buf,"du")
+   || !strcmp(buf,"edit")
    || !strcmp(buf,"find")
    || !strcmp(buf,"recls")
    || !strcmp(buf,"rels")
@@ -717,7 +716,6 @@ static char **lftp_completion (const char *text,int start,int end)
 	 glob_res->Add(f);
       }
       globfree(&pglob);
-      glob_res->ExcludeDots();
 
       rl_filename_completion_desired=1;
       generator = file_generator;
