@@ -6,8 +6,12 @@
 #include <unistd.h>
 #include "FileAccess.h"
 
-int main()
+char *program_name;
+
+int main(int argc,char **argv)
 {
+   program_name=argv[0];
+
    FileAccess *f=FileAccess::New("ftp","ftp.yar.ru");
    if(!f)
    {
