@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2011 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2013 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: misc.h,v 1.55 2011/03/25 14:35:43 lav Exp $ */
-
 #ifndef MISC_H
 #define MISC_H
 
@@ -113,7 +111,10 @@ void base64_encode (const char *s, char *store, int length);
 
 bool temporary_network_error(int e);
 
-CDECL const char *get_lftp_home();
+CDECL const char *get_home();
+CDECL const char *get_lftp_config_dir();
+CDECL const char *get_lftp_data_dir();
+CDECL const char *get_lftp_cache_dir();
 
 const char *memrchr(const char *buf,char c,size_t len);
 static inline char *memrchr(char *buf,char c,size_t len) {
@@ -127,6 +128,8 @@ void rtrim(char *s);
 void random_init();
 double random01();
 
+const char *get_nodename();
+
 #define ListAdd(type,chain,this,next)  \
 do {				       \
    this->next=chain;		       \
