@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: resource.cc,v 1.171 2011/04/22 15:43:11 lav Exp $ */
-
 #include <config.h>
 
 #include <pwd.h>
@@ -215,6 +213,7 @@ static ResType lftp_vars[] = {
    {"ftps:initial-prot",	 "",	  ProtValidate,0},
 #endif
    {"ftp:stat-interval",	 "1",	  ResMgr::TimeIntervalValidate,0},
+   {"ftp:strict-multiline",	 "off",	  ResMgr::BoolValidate,0},
    {"ftp:sync-mode",		 "on",    ResMgr::BoolValidate,0},
    {"ftp:trust-feat",		 "no",	  ResMgr::BoolValidate,0},
    {"ftp:use-abor",		 "yes",   ResMgr::BoolValidate,0},
@@ -236,6 +235,7 @@ static ResType lftp_vars[] = {
    {"ftp:use-telnet-iac",	 "yes",   ResMgr::BoolValidate,0},
    {"ftp:use-tvfs",		 "auto",  ResMgr::TriBoolValidate,0},
    {"ftp:use-ip-tos",		 "no",	  ResMgr::BoolValidate,0},
+   {"ftp:use-utf8",		 "yes",	  ResMgr::BoolValidate,0},
    {"ftp:use-quit",		 "yes",   ResMgr::BoolValidate,0},
    {"ftp:verify-address",	 "no",    ResMgr::BoolValidate,0},
    {"ftp:verify-port",		 "no",    ResMgr::BoolValidate,0},
@@ -253,6 +253,7 @@ static ResType lftp_vars[] = {
    {"hftp:use-head",		 "yes",   ResMgr::BoolValidate,0},
    {"hftp:use-mkcol",		 "no",	  ResMgr::BoolValidate,0},
    {"hftp:use-propfind",	 "no",	  ResMgr::BoolValidate,0},
+   {"hftp:use-allprop",		 "no",	  ResMgr::BoolValidate,0},
    {"hftp:use-type",		 "yes",   ResMgr::BoolValidate,0},
    {"http:accept",		 "*/*",   0,0},
    {"http:accept-language",	 "",	  0,0},
@@ -263,6 +264,7 @@ static ResType lftp_vars[] = {
    {"http:proxy",		 "",	  HttpProxyValidate,0},
    {"http:use-mkcol",		 "yes",   ResMgr::BoolValidate,0},
    {"http:use-propfind",	 "no",    ResMgr::BoolValidate,0},
+   {"http:use-allprop",		 "no",	  ResMgr::BoolValidate,0},
    {"http:user-agent",		 PACKAGE"/"VERSION,0,0},
    {"http:cookie",		 "",	  0,0},
    {"http:set-cookies",		 "no",	  0,0},
@@ -353,6 +355,7 @@ static ResType lftp_vars[] = {
    {"ssl:cert-file",		 "",	  ResMgr::FileReadable,0},
    {"ssl:check-hostname",	 "yes",	  ResMgr::BoolValidate,0},
    {"ssl:verify-certificate",	 "yes",	  ResMgr::BoolValidate,0},
+   {"ssl:use-sni",		 "yes",	  ResMgr::BoolValidate,0},
 # if USE_OPENSSL
    {"ssl:ca-path",		 "",	  ResMgr::DirReadable,ResMgr::NoClosure},
    {"ssl:crl-path",		 "",	  ResMgr::DirReadable,ResMgr::NoClosure},
